/*
** ============================================================================
**
** FILE
**  $Id: IA443x_demo.c,v 1.0 2007/11/29 krk Exp apache $
**
** DESCRIPTION
**  Contains the RF dependent functions
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/
#include <stdio.h>
#include <string.h>
#include "IA443x_demo.h"
#include "global_definitions.h"
#include "IA443x_rf.h"
#include "IA443x_rf_callback.h"
#include "menu.h"
#include "dog_glcd.h"
#include "uart.h"
#include "IAI_EBID.h"


/*------------------------------------------------------------------------*/
/*						GLOBAL variables								  */
/*------------------------------------------------------------------------*/
volatile DEMO_STATES DemoStates;
PACKET_TYPE fWaitForPacketOrAck,fPacketSent;
uint8 rf_param[10], AutoTx, LongDelay;
uint16 NmbrOfSentPackets, NmbrOfReceivedPackets;

PHY_STATUS RfStatus;
uint8 RxTxBuffer[MAX_LENGTH_OF_PAYLOAD];
HEADER HeaderBuffer;
MESSAGE MessageBuffer;
RECEIVE_COMMAND RxCommand;
RECEIVED_DATA ReceivedData;
volatile DEMO_PARAMETERS demo_param;

volatile uint8 customer_name[CUSTOMER_NAME_SIZE];

extern const volatile DEMO_FREQ demo_freqs[MAX_FREQ_SETTING];
extern EBID_DATA TestcardData;

/*------------------------------------------------------------------------*/
/*						LOCAL function prototypes						  */
/*------------------------------------------------------------------------*/
void DemoPingSent(uint8 success, uint16 pid);
void DemoAckSent(uint8 success);
void DemoPingReceived(uint16 pid);
void DemoAckReceived(uint8 success, uint16 pid);

/*------------------------------------------------------------------------*/
/*						LOCAL definitions								  */
/*------------------------------------------------------------------------*/
#define PER_RESET_INPUT		PB2_PIN
#define SEND_PACKET_INPUT 	PB1_PIN
#define LED_TX				LED1_PIN
#define LED_RX 				LED2_PIN
#define LED_ANT1			LED3_PIN
#define	LED_ANT2			LED4_PIN

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void DemoInit(void)
  +
  + DESCRIPTION:    initializes the demo
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void DemoInit(void)
{
	//initialize the demo variables
	DemoStates = sDemoInit;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void DemoStateMachine(void)
  +
  + DESCRIPTION:    state machine of the demo
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void DemoStateMachine(void)
{	
	uint8 temp8,i,string[6];	
	uint16 temp16, delay;
	static uint8 demo_mode;

	switch( DemoStates )
	{
		case sDemoInit:
			//set default variables
			demo_param.datarate = DR10KBPS_DEV5KHZ_MOD1;
			DemoStates = sDemoStartScreen;
			//find out what kind of testcard is plugged into the PIC SW Dev board
			GetEBIDDataFromEEPROM();
			if( TestcardData.valid_info == FALSE )
			{//there is no Testcard inserted or there is no EEPROM on the testcard
				PerformMenu(sMenuTestcardOrEEPROMError);
				DemoStates = sDemoDoNothing;
				return;	
			}
			//find out whether the chip is supported by the current FW or not
			switch( TestcardData.chip_type )
			{
				case IA4432:
					//initialize the RF chip
					temp8 = RfInitHw();
					if( temp8 == RF_ERROR_TIMING )
					{//POR Timeout error
						PerformMenu( sMenuPORTimeout );
						DemoStates = sDemoDoNothing;
						return;
					}
				break;
		
				default:
					PerformMenu(sMenuNotSupportedChip);
					DemoStates = sDemoDoNothing;
					return;	
				break;
			}
		break;

		case sDemoStartScreen:
			//draw the startup screen
			PerformMenu( sMenuStartScreen );
			//start long timeout -> 3s
			LongDelay = 24;
			StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, 62500, FALSE);

			DemoStates = sDemoDelayAfterScreens;
		break;

		case sDemoDelayAfterScreens:
			if( Tmr3Expired() == TRUE )
			{//TMR expired
				StopTmr3();
				if( LongDelay > 0 )
				{//restart if needed for another 250ms
					StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, 62500, FALSE);
					LongDelay--;
				}
				else
				{	
					DemoStates = sDemoInitRf;
				}
			}
			if( (PB1_PIN == 0) || (PB2_PIN == 0) || (PB3_PIN == 0) || (PB4_PIN == 0) )
			{
				while( (PB1_PIN == 0) || (PB2_PIN == 0) || (PB3_PIN == 0) || (PB4_PIN == 0) );
				DemoStates = sDemoInitRf;
			}
		break;

		case sDemoInitRf:
			//initialize settings menu item
			PerformMenu( sMenuInitSettingsPage );
			//next state
			DemoStates = sDemoSettingScreen;
		break;


		case sDemoSettingScreen:
			//stop RF stack and timer
			RFIdle();
			StopTmr3();
			//draw settings screen
			PerformMenu( sMenuSettingsScreen );
			//next state
			DemoStates = sDemoCheckPb;
			//enable LBD to measure the battery voltage
			rf_param[0] = TRUE;
			rf_param[1] = FALSE;		//disable LBD IT
			rf_param[2] = 30;			//threshold is 3.175V
			RFSetParameter(pLowBatteryDetector, rf_param, 3);
		break;

		case sDemoCheckPb:
			//check whether any of the push button is pressed for changing the parameters
			switch( PerformMenu( sMenucheckButtonsForSettings ) )
			{	
				case BUTTON_1:
				case BUTTON_2:
				case BUTTON_3:
					DemoStates = sDemoSettingScreen;
				break;
			
				case BUTTON_4:
					DemoStates = sDemoRestartDemo;
				break;

				default:
				break;
			}
		break;

		case sDemoRestartDemo:
			//draw demo start screen
			PerformMenu( sMenuRestartDemo );
			/**********************************/
			/*initialize RF stack for the demo*/
			/**********************************/
			SetRadio( demo_param.datarate );
			//clear flags and variables
			NmbrOfSentPackets = 0;
			NmbrOfReceivedPackets = 0;
			AutoTx = FALSE;
			LED_TX = 0;
			LED_RX = 0;
			BLED_PIN = 0;
			//next state
			DemoStates = sDemoStartReceive;
		break;

		case sDemoStartReceive:
			//reset the RF stack
			RFIdle();
			//set variables
			if( NmbrOfSentPackets == 999 )
			{
				NmbrOfSentPackets = 1;
				NmbrOfReceivedPackets = 0;
			}
			else
			{
				NmbrOfSentPackets++;

			}
			//set flags -> wait for PING packet and not for ACK
			fWaitForPacketOrAck = PING;
			
			//if auto tx on -> check PB / send next packet
			if( AutoTx == TRUE )
			{
				LongDelay = 1;
				StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, 62500, FALSE);
				DemoStates = sDemoDelayBeforeTx;
				return;
			}
			//start receive
			RFReceive(&RxCommand);
			//next state
			DemoStates = sDemoWaitForReceive;
		break;

		case sDemoDelayBeforeTx:
			//check the push buttons
			if(SEND_PACKET_INPUT == 0)
			{	/*AUTO TRANSMIT FUNCTION IS STOPPED*/
				//wait for release the button
				while(SEND_PACKET_INPUT == 0);
				AutoTx = FALSE;
				DemoStates = sDemoRestartDemo;
				return;
			}
			if( PB3_PIN == 0 || PB4_PIN == 0 )
			{	/*SETTING BUTTON IS PRESSED*/
				//wait for release the button
				while( PB3_PIN == 0 || PB4_PIN == 0 );
				//goto setting page
				DemoStates = sDemoSettingScreen;
				return;				
			}
			if( Tmr3Expired() == TRUE )
			{//TMR expired
				StopTmr3();
				if( LongDelay > 0 )
				{//restart if needed for another 250ms
					StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, 62500, FALSE);
					LongDelay--;
					return;
				}
				else
				{	
					DemoStates = sDemoSendPing;
					return;
				}
			}
		break;

		case sDemoWaitForReceive:
			//check whether the push button is pressed for sending a PING packet
			if( (SEND_PACKET_INPUT == 0) && (fWaitForPacketOrAck == PING) && (AutoTx == FALSE) ) 
			{	/*AUTO TX FUNCTION IS ENABLED*/
				//stop receiving
				RFIdle();
				//wait for release TX push button
				while(SEND_PACKET_INPUT == 0);
				//switch on auto tx
				AutoTx = TRUE;
				//change TXON icon to TXOFF icon
				PerformMenu( sMenuStartAutoTx );
				//clear counters and variables
				NmbrOfSentPackets = 1;
				NmbrOfReceivedPackets = 0;
				//switch off LEDs
				LED_TX = 0;
				LED_RX = 0;
				BLED_PIN = 0;
				//next state
				DemoStates = sDemoSendPing;
				return;
			}
			if( PB3_PIN == 0 || PB4_PIN == 0 )
			{
				while( PB3_PIN == 0 || PB4_PIN == 0 );
				//goto setting page
				DemoStates = sDemoSettingScreen;
				return;				
			}
		break;

		case sDemoPacketReceived:
			//stop timer
			StopTmr3();
			//get received packet and put it into the buffer 
			MessageBuffer.data = &RxTxBuffer[0];
			ReceivedData.message = MessageBuffer;
			RFGetBuffer(&ReceivedData);
			//stop RF stack
			RFIdle();
			//check what kind of packet received
			if( fWaitForPacketOrAck == PING )			
			{//simple packet received -> send an ACK 
				if( (RxTxBuffer[1] == demo_param.self_address) && (RxTxBuffer[4] == PING) )
				{//if the packet is sent to this device, then send ACK
					//report the result
					DemoPingReceived(((uint16)(RxTxBuffer[2] << 8) + (uint16)RxTxBuffer[3]));
					//set ACK packet
					memcpy(&string[0],&RxTxBuffer[0],4);
					RxTxBuffer[0] = string[1];
					RxTxBuffer[1] = string[0];
					RxTxBuffer[2] = string[2];
					RxTxBuffer[3] = string[3];
					RxTxBuffer[4] = ACK;
					MessageBuffer.header = HeaderBuffer;
					MessageBuffer.length = 5;
					MessageBuffer.data = &RxTxBuffer[0];
					//send the packet
					LED_TX = 1;
					RFTransmit(&MessageBuffer);
					//set flag ->ACK packet has sent
					fPacketSent = ACK;
					//next state
					DemoStates = sDemoWaitForSendPacket;	
				}
				else
				{//the packet is not PING or sent to another device, so discard it -> goto RX state: 
				 //wait for push button or another PING packet
					//goto RX state: wait for push button or another packet
					DemoStates = sDemoStartReceive;
				}
				return;
			}
			if( fWaitForPacketOrAck == ACK )
			{//ACK packet received -> calculate the PER and print the result	
				//report the result
				DemoAckReceived(TRUE, NmbrOfSentPackets);
				//goto RX state: wait for push button or another PING packet
				DemoStates = sDemoStartReceive;
				return;
			}
		break;

		case sDemoRxError:
			//stop RF stack and timer
			RFIdle();
			StopTmr3();
			if( fWaitForPacketOrAck == PING )
			{//error occured during receiving a PING packet
				DemoStates = sDemoStartReceive;
			}
			if( fWaitForPacketOrAck == ACK )
			{//error occured during receiving an ACK packet
				//report that ACK didn't receive
				DemoAckReceived(FALSE, NmbrOfSentPackets);
				//goto RX state: wait for push button or another PING packet
				DemoStates = sDemoStartReceive;
			}
		break;

		case sDemoSendPing:
			//set the payload of PING message
			RxTxBuffer[0] = demo_param.self_address;				//source address field
			RxTxBuffer[1] = demo_param.destination_address;			//destination address field
			RxTxBuffer[2] = (uint8)(NmbrOfSentPackets >> 8);		//16bit packet ID (incremented after every
			RxTxBuffer[3] = (uint8)(NmbrOfSentPackets & 0x00FF);	//		PING packet transmission
			RxTxBuffer[4] = PING;									//packet type field
			//set the PING message
			MessageBuffer.header = HeaderBuffer;
			MessageBuffer.length = 5;								
			MessageBuffer.data = &RxTxBuffer[0];
			//start sending the PING packet
			LED_TX = 1;
			RFTransmit(&MessageBuffer);
			//set flag ->PING packet sent
			fPacketSent = PING;
			//next state
			DemoStates = sDemoWaitForSendPacket;
		break;

		case sDemoWaitForSendPacket:
			//do nothing, wait for the RF stack to send the packet
		break;

		case sDemoPacketSent:
			//check what kind of packet has sent
			LED_TX = 0;
			if( fPacketSent == PING )
			{//PING packet sent -> goto RX state immediatelly: wait for ACK
				//stop the RF stack
				RFIdle();
				//set flags -> wait for simple packet and not for ACK
				fWaitForPacketOrAck = ACK;
				//start receiving
				RFReceive(&RxCommand);
				//start timeout (100ms is enough even at 1.2kbps data rate)
				StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, ACK_TIMEOUT, FALSE);
				//next state
				DemoStates = sDemoWaitForAck;
			}
			if( fPacketSent == ACK )
			{//ACK packet sent -> goto RX state: wait for push button or another PING packet
				//stop the RF stack
				RFIdle();
				//report that ACK sent correctly
				DemoAckSent(TRUE);	
				//next state
				DemoStates = sDemoStartReceive;
			}
		break;
		
		case sDemoTxError:
			//error occured during packet transmission
			LED_TX = 0;
			if( fPacketSent == PING)
			{//ping packet didn't send, because error occured
				DemoPingSent(FALSE, NmbrOfSentPackets);
				DemoStates = sDemoStartReceive;
			}
			if( fPacketSent == ACK )	
			{//ACK packet didn't send, because error occured
			}
		break;

		case sDemoWaitForAck:
			if( Tmr3Expired() == TRUE )
			{//ACK didn't receive within timeout
				//stop timer & RF Stack
				StopTmr3();
				RFIdle();
				//report that timeout occured and ACK didn't receive
				DemoAckReceived(FALSE, NmbrOfSentPackets);
				//goto RX state: wait for push button or another PING packet
				DemoStates = sDemoStartReceive;
			}
		break;		

		case sDemoDoNothing:
		default:
		break;
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void DemoPingSent(uint8 success, uint16 pid)
  +
  + DESCRIPTION:    the function is called if the PING packet sent correctly
  +					or there was a problem to send it. It reports the result
  +					on the LCD and to the UART.
  +
  +	INPUT:			success - status of the PING transmit
  +					pid		- packet ID of the transmitted PING packet
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void DemoPingSent(uint8 success, uint16 pid)
{
	if( success == TRUE )
	{//packet sent correctly

	}
	else
	{//error occured during packet transmission
		PerformMenu( sMenuPingTxError );
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void DemoAckSent(uint8 success)
  +
  + DESCRIPTION:    the function is called if the ACK packet sent correctly
  +					or there was a problem to send it. It reports the result
  +					on the LCD and to the UART.
  +
  +	INPUT:			success - status of the ACK transmition (TRUE/FALSE)
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void DemoAckSent(uint8 success)
{
	if( success == TRUE )
	{//packet sent correctly
		LED_TX = 1;
		PerformMenu( sMenuAckSent );
		LED_TX = 0;
	}
	else
	{//error occured during packet transmission
		PerformMenu( sMenuAckTxError	 );
	}
	//timeout to show the result
	StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, 25000, FALSE);
	while( Tmr3Expired() == FALSE );	
	StopTmr3();
	PerformMenu( sMenuDeleteMessageRow );
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void DemoPingReceived(uint16 pid)
  +
  + DESCRIPTION:    the function is called if a PING packet received correctly
  +					It reports the result on the LCD and to the UART.
  +
  +	INPUT:			pid		- packet ID of the received PING packet
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void DemoPingReceived(uint16 pid)
{
	LED_RX = 1;
	BLED_PIN = 1;
	//print RSSI result
	PerformMenu(sMenuPingReceived);
	StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, 2500, FALSE);
	while( Tmr3Expired() == FALSE );	
	StopTmr3();
	LED_RX = 0;
	BLED_PIN = 0;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void DemoAckReceived(uint8 success, uint16 pid)
  +
  + DESCRIPTION:    the function is called if an ACK packet received correctly
  +					or there was a problem during the ACK reception.
  +					It reports the result on the LCD and to the UART.
  +
  +	INPUT:			success - status of the ACK reception (TRUE/FALSE)
  +					pid		- packet ID of the received ACK
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void DemoAckReceived(uint8 success, uint16 pid)
{
uint8 i;
float temp_float;

	//set the variable
	if( success == TRUE )
	{
		LED_RX = 1;
		BLED_PIN = 1;
		NmbrOfReceivedPackets++;
		PerformMenu( sMenuAckReceivedMessage );
	}
	else
	{
		PerformMenu(sMenuAckReceivingErrorMessage);
	}
	//calc the PER
	temp_float = CalcPer(NmbrOfSentPackets,NmbrOfReceivedPackets);
	
	//report the result
	MenuReportPer(NmbrOfSentPackets,NmbrOfReceivedPackets,temp_float);

	LED_RX = 0;
	BLED_PIN = 0;
	StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, 50000, FALSE);
	while( Tmr3Expired() == FALSE );	
	StopTmr3();
	PerformMenu( sMenuDeleteMessageRow);
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  float CalcPer(uint16 total, uint16 received)
  +
  + DESCRIPTION:    calculate the PER
  +	
  + RETURN:         PER in percentage
  +
  +	INPUT:			total    - total number of expected packages
  +					received - number of received packages
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
float CalcPer(uint16 total, uint16 received)
{
	float temp_float;

	temp_float = 0;

	if( total != 0 )
	{
		temp_float = ((float)(total - received) / (float)total) * 100;		
		if(temp_float > 100)
		{
			temp_float = 100;
		}
	}

	return temp_float;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void SetRadio(void)
  +
  + DESCRIPTION:    set radio parameters
  +	
  + RETURN:         Freq number
  +
  +	INPUT:			data rate
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 SetRadio(uint8 data_rate)
{
uint8 temp8;
uint16 temp16;

	//wake up the RF chip if it was in standby state
	temp8 = RFWakeUp();
	if( temp8 == RF_OK )
	{//wait for waking up	
		do{RFGetStatus(&RfStatus);} while( RfStatus.status == sRFWakingUp );
		RFGetStatus(&RfStatus);
		if( RfStatus.status == sRFWakeUpError )
		{
			PerformMenu( sMenuXTALError );
		}
	}
	else
	{//reset the RF stack
		RFIdle();
	}
	demo_param.frequency_band = demo_freqs[demo_param.freq_number].frequency_band;
	demo_param.frequency = demo_freqs[demo_param.freq_number].frequency;

	rf_param[0] = demo_param.frequency_band;
	rf_param[1] = (uint8)(demo_param.frequency >> 8);
	rf_param[2] = (uint8)(demo_param.frequency & 0x00FF);
	RFSetParameter(pRfFrequency, rf_param, 3);

	//header fields are not used in the packet
	HeaderBuffer.enabled_headers = 0;
	RxCommand.header = HeaderBuffer;
	//preamble detection
	rf_param[0] = PD_LENGTH;
	rf_param[1] = PD_ERROR;
	RFSetParameter(pRfPremableDetection, rf_param, 2);
	//synch word
	rf_param[0] = 2;
	rf_param[1] = 0x2D;
	rf_param[2] = 0xD4;
	RFSetParameter(pRfSynchronWord, rf_param, 3);
	//GPIOs
	switch( TestcardData.ant_type )
	{
		case ANT_DIVERSITY:
			LED4_PIN = 1;
			rf_param[0] = TRUE;
			RFSetParameter(pRfAntennaDiversity, rf_param, 1);
			//ANT diversity - GPIO1 & 2 is ANT DIV switch signals
			rf_param[0] = GPIO_GND;
			rf_param[1] = GPIO_ANT1_SWITCH;
			rf_param[2] = GPIO_ANT2_SWITCH;
			LED4_PIN = 0;
		break;

		case SMA_TRX_SWITCH:
			rf_param[0] = FALSE;
			RFSetParameter(pRfAntennaDiversity, rf_param, 1);
			//TRX switch control - GPIO1: TX_STATE, GPIO2: RX_STATE 
			rf_param[0] = GPIO_GND;
			rf_param[1] = GPIO_TX_STATE;
			rf_param[2] = GPIO_RX_STATE;
		break;

		case SEPARATE_SMA:
		default:
			rf_param[0] = FALSE;
			RFSetParameter(pRfAntennaDiversity, rf_param, 1);
			//separate RX & TX SMA connector - GPIOs are not used -> ground them
			rf_param[0] = GPIO_GND;
			rf_param[1] = GPIO_GND;
			rf_param[2] = GPIO_GND;
		break;
	}
	RFSetParameter(pRfGpios, rf_param, 3);
	//set rf parameters according the requirements
	RFSetRfParameters( data_rate );	

	return demo_param.freq_number;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint16 GetFreq(uint8 freq_number)
  +
  + DESCRIPTION:    it gives back the center freq in decimal format
  +	
  + RETURN:         frequency in decimal
  +
  +	INPUT:			serial number of the frequency setting
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint16 GetFreq(uint8 freq_number)
{
	return demo_freqs[freq_number].freq_decimal;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void GetAntDiversityResult(void)
  +
  + DESCRIPTION:    switch the ANT LEDs according the RSSI on the antennas 
  +	
  + RETURN:         
  +
  +	INPUT:			
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void GetAntDiversityResult(void)
{
	uint8 RssiAnt1, RssiAnt2;
	
	//get RSSI info from the PHY layer
	RFGetRssiResult(&RssiAnt1, &RssiAnt2);
	//switch the LEDs accordingly
	switch( TestcardData.ant_type )
	{
		//antenna div used -> find out which is the stronger signal
		case ANT_DIVERSITY:
			if( RssiAnt1 > RssiAnt2 )
			{//the reception is done on ANT1
				LED_ANT1 = 1;
				LED_ANT2 = 0;
			}
			else
			{//the reception is done on ANT2
				LED_ANT1 = 0;
				LED_ANT2 = 1;
			}
		break;

		//no diversity is used, show ANT1 LED
		default:
			LED_ANT1 = 1;
			LED_ANT2 = 0;
		break;
	}
}
