/*
** ============================================================================
**
** FILE
**  $Id: IA443x_rf.c,v 1.0 2007/11/28 krk Exp apache $
**
** DESCRIPTION
**  Contains the RF dependent functions
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#include "IA443x_rf.h"
#include "IA443x_rf_callback.h"
#include "pic18_timers.h"
#include "IAI_EBID.h"

/*------------------------------------------------------------------------*/
/*						GLOBAL variables								  */
/*------------------------------------------------------------------------*/
volatile RF_ENUM RfState;
volatile uint8 ItStatus1, ItStatus2, Rssi_ant1, Rssi_ant2, ByteTime, fPacketReceived;
uint16 PdTimeout;
const uint8 RfSettings[NMBR_OF_SAMPLE_SETTING][NMBR_OF_PARAMETER] =		//revX2
{
//	 IFBW, COSR, CRO2, CRO1, CRO0, CTG1, CTG0, TDR1, TDR0, FDEV, B_TIME

	{0x17, 0x41, 0x60, 0x27, 0x52, 0x00, 0x02, 0x00, 0x9D, 0x39, 209},	//DR2400BPS_DEV36KHZ 
	{0x03, 0x41, 0x60, 0x27, 0x52, 0x00, 0x10, 0x01, 0x3B, 0x48, 105},	//DR4800BPS_DEV45KHZ
	{0x03, 0xA1, 0x20, 0x4E, 0xA5, 0x00, 0x36, 0x02, 0x75, 0x48, 53 },	//DR9600BPS_DEV45KHZ
	{0x21, 0x64, 0x01, 0x47, 0xAE, 0x07, 0xFF, 0x02, 0x8F, 0x08, 50 },	//DR10KBPS_DEV5KHZ_MOD1
	{0x11, 0x64, 0x01, 0x47, 0xAE, 0x07, 0xFF, 0x05, 0x1F, 0x10, 25 },	//DR20K0PS_DEV10KHZ_MOD1
	{0x01, 0x64, 0x01, 0x47, 0xae, 0x07, 0xFF, 0x0A, 0x3D, 0x20, 13	},	//DR40KBPS_DEV20KHZ_MOD1
	{0x04, 0x50, 0x01, 0x99, 0x9a, 0x07, 0xFF, 0x0C, 0xCD, 0x28, 10	},	//DR50KBPS_DEV25KHZ_MOD1
	{0x81, 0x78, 0x01, 0x11, 0x11, 0x07, 0xFF, 0x19, 0x9A, 0x50, 5	},	//DR100KBPS_DEV100KHZ_MOD1	
	{0x82, 0x60, 0x01, 0x55, 0x55, 0x07, 0xFF, 0x20, 0x00, 0x64, 4	},	//DR125KBPS_DEV62KHZ5_MOD1
};

extern EBID_DATA TestcardData;

/*------------------------------------------------------------------------*/
/*						GLOBAL function prototypes						  */
/*------------------------------------------------------------------------*/
uint16 GetFrequency(uint8 channel_id);

/*------------------------------------------------------------------------*/
/*						LOCAL function prototypes						  */
/*------------------------------------------------------------------------*/
void RFStateMachine(RF_ENUM ItSource);

/*------------------------------------------------------------------------*/
/*						LOCAL definitions								  */
/*------------------------------------------------------------------------*/
//#define RF_LED_DEBUG	

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void RfInitHw(void)
  +
  + DESCRIPTION:    initializes the used I/O pins, SPI and timer peripherals, 
  +					IT routines needed for the RF stack
  +
  + RETURN:         None
  +
  + NOTES:          1.) has to be called in the power-on routine
  +					2.) it doesn't initialize the RF chip registers
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RfInitHw(void)
{
	uint8 PorIt;

	//initialize I/O port directions
	RF_NIRQ_DIR 	= INPUT;
	RF_NSEL_DIR 	= OUTPUT;
	RF_GPIO0_DIR	= INPUT;
	RF_GPIO1_DIR	= INPUT;	
	RF_GPIO2_DIR	= INPUT;
	EE_NSEL_DIR		= OUTPUT;
	
	RF_NSEL_PIN 	= 1;
	EE_NSEL_PIN		= 1;

	//TMR0 is used for the RF stack
	DisableTmr0It();

	//set MAC EXT IT - INT0 is used for this purpose (falling edge)
	SetExt0ItFallingEdge();
	DisableExt0It();

	//setup RF stack state machine
	RfState = sRFPOR;
	//release all IT flag to set IRQ high
	ItStatus1 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus1 << 8));
	ItStatus2 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus2 << 8));

	//SW reset	-> wait for POR interrupt
	PorIt = FALSE;
	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), 0x80);
	//timeout for POR interrupt
	StartTmr0(TMR0_16BIT,RESET_TIMEOUT_DIV,RESET_TIMEOUT,FALSE);
	do{
		if( RF_NIRQ_PIN == 0 )
		{
			PorIt = TRUE;
		}
	}while( (Tmr0Expired() == FALSE) && (PorIt == FALSE) );
	//check the status of the POR
	if( RF_NIRQ_PIN == 0x00 )	
	{//POR performed correctly
		//disable XTAL
		SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), 0x00);
		//set RF stack state
		RfState = sRFStandBy;
		DisableTmr0It();
	}
	else
	{//POR error
		DisableTmr0It();
		return RF_ERROR_TIMING;
	}

	//release all IT flag
	ItStatus1 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus1 << 8));
	ItStatus2 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus2 << 8));
	//disable all ITs, except 'ichiprdy'
	SpiWriteAddressData((REG_WRITE | InterruptEnable1), 0x00);
	SpiWriteAddressData((REG_WRITE | InterruptEnable2), 0x02);
	
	//set the special registers (what has to be different than the POR-default)
	SetSpecialRegisters();

	//Set TX Ramp Control
	SpiWriteAddressData((REG_WRITE | TXRampControl), 0x7F);
	//disable AFC
	SpiWriteAddressData((REG_WRITE | AFCLoopGearshiftOverride), 0x00);
	//reset digital testbus, disable scan test
	SpiWriteAddressData((REG_WRITE | DigitalTestBus), 0x00);		
	//select nothing to the Analog Testbus
	SpiWriteAddressData((REG_WRITE | AnalogTestBus), 0x0B);

	return RF_OK;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFWakeUp(void)
  +
  + DESCRIPTION:    wakes up the RF chip from STANDBY.
  +
  + RETURN:         RF_OK: 		 	the operation was succesfull
  +                 RF_ERROR_STATE: the operation is ignored, because the 
  +									RF stack was not in STANDBY state.
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFWakeUp(void)
{
	uint8 temp8;

	if( RfState != sRFStandBy )
		return RF_ERROR_STATE;

	//start the crystal oscillator
	temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
	temp8 |= 0x01;
 	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);

	//set RF stack state
	RfState = sRFWakingUp;

	//set timeout for XTAL startup
	StartTmr0(TMR0_16BIT,XTAL_WAKE_UP_PERIOD_DEV,XTAL_WAKE_UP_PERIOD,TRUE);
	EnableExt0It();

	return RF_OK;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFSleep(void)
  +
  + DESCRIPTION:    sets the transceiver into STANDBY state.
  +
  + RETURN:         RF_OK:			 the operation was succesfull
  +                 RF_ERROR_STATE:  the operation is ignored, because the 
  +									 RF stack was not in IDLE state.
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFSleep(void)
{
	uint8 temp8;

	if( RfState != sRFIdle )
		return RF_ERROR_STATE;

	//stop the crystal oscillator
	temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
	temp8 &= 0xFE;
    SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);
	//set next state
	RfState = sRFStandBy;

	return RF_OK;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFSetParameter(RF_PARAMETER parameter, uint8 * value, uint8 length)
  +
  + DESCRIPTION:    sets the appropiate registers
  +
  +	INPUT:			parameter: 		 name of the RF parameter needs to be change
  +					value:			 value of the parameter (it maybe more bytes)
  +         		length:			 number of bytes of the value
  + 				
  + RETURN:         RF_OK:			 	the operation was succesfull
  +                 RF_ERROR_STATE:  	the operation is ignored, because the 
  +									 	RF stack was not in IDLE state.
  +					RF_ERROR_PARAMETER: the input paramter is invalid, operation is ignored
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFSetParameter(RF_PARAMETER parameter, uint8 * value, uint8 length)
{
uint8 temp8, i;

	if( RfState != sRFIdle )
		return RF_ERROR_STATE;

	switch(parameter)
	{
		/*
		 value[0] - frequency band
		 value[1] - Carrier frequency1
		 value[2] - Carrier frequency2
		*/
		case pRfFrequency:
			//set frequency band
			SpiWriteAddressData((REG_WRITE | FrequencyBandSelect), *value++);						
			//set frequency
			SpiWriteAddressData((REG_WRITE | NominalCarrierFrequency1), *value++);
			SpiWriteAddressData((REG_WRITE | NominalCarrierFrequency0), *value);
		break;

		/*
		 value[0] - GPIO0
		 value[1] - GPIO1
		 value[2] - GPIO2
		*/
		case pRfGpios:
		 	SpiWriteAddressData((REG_WRITE | GPIO0Configuration), *value++);
		 	SpiWriteAddressData((REG_WRITE | GPIO1Configuration), *value++);
		 	SpiWriteAddressData((REG_WRITE | GPIO2Configuration), *value++);
		break;

		/*
		 value[0] - DIGITAL OUT: bit<0> - GPIO0, bit<1> - GPIO1, bit<2> - GPIO2     
		*/
		case pRfDigitalOut:
			SpiWriteAddressData((REG_WRITE | IOPortConfiguration), (*value & 0x07));
		break;

		/*
  		 value[0] - DIGITAL INPUT: bit<0> - GPIO0, bit<1> - GPIO1, bit<2> - GPIO2 
		 value[1] - INTERRUPT INPUT STATE: bit<0> - GPIO0, bit<1> - GPIO1, bit<2> - GPIO2 
		*/
		case pRfDigitalIn:
			temp8 = (uint8)(0x00FF & SpiReadWriteWord(IOPortConfiguration << 8));
			*value++ = temp8 & 0x07;
			*value = (temp8 >> 4) & 0x07;
		break;


		/*
		 value[0] - Output power
		*/
		case pRfTransmitPower:
			SpiWriteAddressData((REG_WRITE | TXPower), *value);
		break;

		/*
		 value[0] - Preamble length in nibbles: bit[8]
		 value[1] - Preamble length in nibbles: bit[7..0]
		*/
		case pRfPreambleLength:
			//set MSB if needed
			temp8 = (uint8)(0x00FF & SpiReadWriteWord(HeaderControl2 << 8));
			if( *value++ == 0)
			{
				temp8 &= 0xFE;
			}
			else
			{
				temp8 |= 0x01;
			}
			SpiWriteAddressData((REG_WRITE | HeaderControl2), (temp8 & 0x07));	//bug in revX: if txhdlen[2:0] is not 0, the length handled wrong way during RX!
			//set lower byte
			SpiWriteAddressData((REG_WRITE | PreambleLength), *value);		
		break;

		/*
		 value[0] - Number of nibbles needs to be match (0...15)
		 value[1] - number of error (0 ... 7)
		*/
		case pRfPremableDetection:
			temp8 = (*value++ & 0x1F) << 3;
			temp8 += (*value & 0x07);
			SpiWriteAddressData((REG_WRITE | PreambleDetectionControl), temp8 );				
		break;
		
		/*
		 value[0] - Length of the synchron word (1...4)
		 value[1] - synchron word 3
		 value[2] - synchron word 2	-if enabled-
		 value[3] - synchron word 1 -if enabled-
		 value[4] - synchron word 0 -if enabled-
		*/
		case pRfSynchronWord:
			temp8 = ((*value++ - 1) << 1) & 0x06;
			i = ((uint8)(0x00FF & SpiReadWriteWord(HeaderControl2 << 8))) & 0x00;	//bug in revX: if txhdlen[2:0] is not 0, the length handled wrong way during RX!
			i += temp8;
			SpiWriteAddressData((REG_WRITE | HeaderControl2), i);						
			for(i=0;i<temp8;i++)
			{
				SpiWriteAddressData((REG_WRITE | (SyncWord3 + i)), *value++);
			}
		break;

		/*
		 value[0] - new value of the modulation mode ctrl 2 register. 
					(Refer to the IA443x_rf.h for available definitions!)
		*/
		case pRfModulation:
			SpiWriteAddressData((REG_WRITE | ModulationModeControl1), 0x00);	//TODO: implement Manchester, etc.		
			SpiWriteAddressData((REG_WRITE | ModulationModeControl2), *value);
		break;

		/*
		 value[0] - wheather the TX / RX packet handler, CRC is enabled or not . 
					(Refer to the IA443x_rf.h for available definitions!)
		*/
		case pRfPacketHandler:
			SpiWriteAddressData((REG_WRITE | DataAccessControl), *value);
		break;

		/*
		 value[0] - whether it uses antenna diversity algorithm or not
		*/
		case pRfAntennaDiversity:
			temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl2 << 8));
			temp8 &= 0x1F;
			if( *value == TRUE)
			{//enable antenna diversity algorithm
				temp8 |= 0x80;
				SpiWriteAddressData((REG_WRITE | OperatingFunctionControl2), temp8);
			}
			else
			{//disable antenna diversity algorithm
				SpiWriteAddressData((REG_WRITE | OperatingFunctionControl2), temp8);
			}
		break;

		/*
		 value[0] - TRUE: enable the LBD; FALSE: disable the LBD
		 value[1] - TRUE: enable IT, FALSE: disable IT
		 value[2] - LBD Threshold
		*/
		case pLowBatteryDetector:
			temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
			if(*value++ == TRUE)
			{//enable the Low Battery Detector
				temp8 |= 0x40;
				SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);
				temp8 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus2 << 8));
				if(*value++ == TRUE)
				{//enable LBD IT
					temp8 |= 0x04;
				}
				else
				{//disable LBD IT
					temp8 &= 0xFB;
				}
				SpiWriteAddressData((REG_WRITE | InterruptStatus2), temp8);
				SpiWriteAddressData((REG_WRITE | LowBatteryDetectorThreshold), *value);
			}
			else
			{//disable the Low Battery Detector
				temp8 &= 0xBF;
				SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);
			}
		break;

		default:	
			return RF_ERROR_PARAMETER;
	}

	return RF_OK;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFSetRfParameters(RF_SAMPLE_SETTINGS setting)
  +
  + DESCRIPTION:    this function configures the RF part of the chip (both TX and RX)
  + 				for different (predefined) data rate, deviation and modulation index 
  +					requirements.
  +					The available settings are (this is the input parameter of the function too):
  +					DR2400BPS_DEV36KHZ			DR = 2.4kbps;	Fdev = 36kHz; 	MOD index = 30;  	BW = 69.2kHz;
  +					DR4800BPS_DEV45KHZ			DR = 4.8kbps;	Fdev = 45kHz; 	MOD index = 18.8;  	BW = 90kHz;
  +					DR9600BPS_DEV45KHZ			DR = 9.6kbps; 	Fdev = 45kHz;   MOD index = 9.4;   	BW = 90kHz;
  +					DR10KBPS_DEV5KHZ_MOD1		DR = 10kbps;	Fdev = 5kHz; 	MOD index = 1; 		BW = 18.9kHz
  +					DR20K0PS_DEV10KHZ_MOD1		DR = 20kbps;	Fdev = 10kHz; 	MOD index = 1; 		BW = 37.7kHz
  +					DR40KBPS_DEV20KHZ_MOD1		DR = 40kbps;	Fdev = 20kHz; 	MOD index = 1; 		BW = 75.2kHz
  +					DR50KBPS_DEV25KHZ_MOD1		DR = 50kbps;	Fdev = 25kHz; 	MOD index = 1; 		BW = 95.3kHz
  +					DR100KBPS_DEV50KHZ_MOD1		DR = 100kbps;	Fdev = 50kHz; 	MOD index = 1; 		BW = 225.6kHz
  + 				DR125KBPS_DEV62KHZ5_MOD1	DR = 125kbps;	Fdev = 62.5kHz; MOD index = 1; 		BW = 149.6kHz
  +
  + RETURN:         RF_OK: 				the operation was succesfull
  +                 RF_ERROR_PARAMETER: invalid parameter, the operation is ignored.
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFSetRfParameters(RF_SAMPLE_SETTINGS setting)
{
	uint8 temp8;

	//check whether the input parameter valid or not
	if( setting > (NMBR_OF_SAMPLE_SETTING - 1))
	{
		return RF_ERROR_PARAMETER;
	}

	//set RF chip for FIFO operation
	//Modulation mode
	SpiWriteAddressData((REG_WRITE | ModulationModeControl1), 0x00);			
	SpiWriteAddressData((REG_WRITE | ModulationModeControl2), (TX_CLK_ON_GPIO | MOD_FIFO | MOD_TYPE_GFSK));
	//enable packet handler & CRC16
	SpiWriteAddressData((REG_WRITE | DataAccessControl), (RX_PACKET_HANDLER_EN | TX_PACKET_HANDLER_EN | CRC_16));

	//set the registers according the selected RF settings
	SpiWriteAddressData((REG_WRITE | IFFilterBandwidth), RfSettings[setting][0] );
	SpiWriteAddressData((REG_WRITE | ClockRecoveryOversamplingRatio), RfSettings[setting][1]);
	SpiWriteAddressData((REG_WRITE | ClockRecoveryOffset2), RfSettings[setting][2]);
	SpiWriteAddressData((REG_WRITE | ClockRecoveryOffset1), RfSettings[setting][3]);
	SpiWriteAddressData((REG_WRITE | ClockRecoveryOffset0), RfSettings[setting][4]);
	SpiWriteAddressData((REG_WRITE | ClockRecoveryTimingLoopGain1), RfSettings[setting][5]);
	SpiWriteAddressData((REG_WRITE | ClockRecoveryTimingLoopGain0), RfSettings[setting][6]);
	SpiWriteAddressData((REG_WRITE | TXDataRate1), RfSettings[setting][7]);
	SpiWriteAddressData((REG_WRITE | TXDataRate0), RfSettings[setting][8]);
	SpiWriteAddressData((REG_WRITE | FrequencyDeviation), RfSettings[setting][9]);
	SpiWriteAddressData((REG_WRITE | ClockRecoveryGearshiftOverride), 0x13);

	//automatic Gain Control (ONLY FOR revX2)
	SpiWriteAddressData((REG_WRITE | AGCOverride1), 0x20);
	//override ADC refernce voltage to 0.8V
	SpiWriteAddressData((REG_WRITE | DeltasigmaADCTuning2), 0x03);

	//store how long does it take to send/receive 1 byte on the given data rate
	//the value represent the Timer setting, assuming, that the clock freq is 8MHz, and the timer prescaler is 32
	ByteTime = RfSettings[setting][10];
	return RF_OK;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFIdle(void)
  +
  + DESCRIPTION:    sets the transceiver and the RF stack into IDLE state, 
  +					independently of the actual state of the RF stack.
  +
  + RETURN:         RF_OK: 			the operation was succesfull
  +                 RF_ERROR_STATE: the actualy state of the stack is aborted.
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFIdle(void)
{
	uint8 temp8;

	if(RfState != sRFIdle)
	{	//switch off everything except XTAL
		temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
		temp8 &= 0xF1; 
		SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);

		//diasble all ITs
		SpiWriteAddressData((REG_WRITE | InterruptEnable1), 0x00);
		SpiWriteAddressData((REG_WRITE | InterruptEnable2), 0x00);
		//release all IT flag
		ItStatus1 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus1 << 8));
		ItStatus2 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus2 << 8));

		//disable ITs
		DisableExt0It();
		StopTmr0();
	
		//set next state
		RfState = sRFIdle;
	}
	return RF_OK;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RfDirectTransmit(direct_transmit_command_t * Struct)
  +
  + DESCRIPTION:    Direct transmit
  +
  + RETURN:         RF_OK: 			the operation was succesfull
  +                 RF_ERROR_STATE: the actualy state of the stack is aborted.
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFDirectTransmit(void)
{	//enable transmitter
	uint8 temp8;

	temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
	temp8 |= 0x09;
	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);

	//disable all the ITs
	SpiWriteAddressData((REG_WRITE | InterruptEnable1), 0x00);
	SpiWriteAddressData((REG_WRITE | InterruptEnable2), 0x00);
	//release all IT flag
	ItStatus1 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus1 << 8));
	ItStatus2 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus2 << 8));

	//disable EXT IT	
	DisableExt0It();
	//disable PHY timer
	StopTmr0();

	//set next state
	RfState = sRFPacketSent;

	return RF_OK;	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RfTransmit(MESSAGE * Struct)
  +
  + DESCRIPTION:    starts packet transmission
  +
  + INPUT:			MESSAGE structure
  +
  + RETURN:         RF_OK: 				the operation was succesfull
  +                 RF_ERROR_STATE: 	the command is ignored, because the MAC 
  +										was not in STANDBY or IDLE state.
  +					RF_ERROR_TIMING: 	the command is ignored, there is 
  +										not enough time to wake up
  +					RF_ERROR_PARAMETER: the command is ignored, some of the 
  +										input parameter(s) are
  +										out of the valid range
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFTransmit(MESSAGE * Struct)
{
uint8 temp8;
uint16 timeout;

	//clear FIFO
	temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl2 << 8));
	temp8 |= 0x01;
	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl2), temp8);
	temp8 &= 0xFE;
	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl2), temp8);

	//set headers and filters
	//bug in revX: if txhdlen[2:0] is not 0, the length handled wrong way during RX!
	if( Struct -> header.enabled_headers == 0 )
	{//no TX header is enabled
		temp8 = (uint8)(0x00FF & SpiReadWriteWord(HeaderControl2 << 8));
		temp8 &= 0x07;
		SpiWriteAddressData((REG_WRITE | HeaderControl2 ), temp8);
	}
	else
	{//TODO: header and header filter are enabled - NOT YET IMPLEMENTED
	}

	//set packet content
	SpiWriteAddressData((REG_WRITE | TransmitPacketLength), Struct -> length - 1);					
	for(temp8=0;temp8<Struct -> length;temp8++)
	{
		SpiWriteAddressData((REG_WRITE | FIFOAccess),Struct -> data[temp8]);		
	}

	//set preamble length (Rev X2)
	SpiWriteAddressData((REG_WRITE | PreambleLength), (PREAMBLE_LENGTH << 1));		
	
	//enable transmitter
	temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
	temp8 |= 0x09;
	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);

	//enable the wanted ITs
	SpiWriteAddressData((REG_WRITE | InterruptEnable1), 0x04);
	SpiWriteAddressData((REG_WRITE | InterruptEnable2), 0x00);
	//release all IT flag
	ItStatus1 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus1 << 8));
	ItStatus2 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus2 << 8));

	//enable EXT IT
	EnableExt0It();
	//start timeout for packet receiving 
	timeout = ByteTime * TX_PACKET_LENGTH;
	StartTmr0(TMR0_16BIT,BYTE_TIME_DIV,timeout,TRUE);

	//set next state
	RfState = sRFTransmitting;

	return RF_OK;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFDirectReceive(void)
  +
  + DESCRIPTION:    starts direct reception
  +
  + INPUT:			None
  +
  + RETURN:         RF_OK: 				the operation was succesfull
  +                 RF_ERROR_STATE: 	the command is ignored, because the MAC 
  +										was not in STANDBY or IDLE state.
  +					RF_ERROR_TIMING: 	the command is ignored, there is
  +									 	not enough time to wake up
  +					RF_ERROR_PARAMETER: the command is ignored,some of the 
  +										input parameter(s) are
  +										out of the valid range
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFDirectReceive(void)
{
	uint8 temp8;

	if( RfState != sRFIdle )
		return RF_ERROR_STATE;

	//SpiWriteAddressData((REG_WRITE | ClockRecoveryGearshiftOverride), 0x52);
	//restart ADC
	SpiWriteAddressData((REG_WRITE | DeltasigmaADCTuning1), 0x80);
	SpiWriteAddressData((REG_WRITE | DeltasigmaADCTuning1), 0x00);

	//set preamble length (Rev X2)
	SpiWriteAddressData((REG_WRITE | PreambleLength), ((PREAMBLE_LENGTH << 1) + EXTRA_PREA));		


	//enable receiver chain
	temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
	temp8 |= 0x05;
	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);	

	//these enable to see the RX signal on the spectrum analyzer
	#ifdef CHECK_RX_ON_SPA
	 SpiWriteAddressData((REG_WRITE | BlockEnableOverride2), 0xAC);
	 SpiWriteAddressData((REG_WRITE | BlockEnableOverride1), 0x10);
     SpiWriteAddressData((REG_WRITE | LDOControlOverride), 0x08);
	#endif

	//disable all the ITs
	SpiWriteAddressData((REG_WRITE | InterruptEnable1), 0x00);
	SpiWriteAddressData((REG_WRITE | InterruptEnable2), 0x00);
	//release all IT flag
	ItStatus1 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus1 << 8));
	ItStatus2 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus2 << 8));

	//enable EXT IT
	DisableExt0It();
	//disable PHY timer
	StopTmr0();

	//set next state
	RfState = sRFPacketReceived;
	
	fPacketReceived = FALSE;

	return RF_OK;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFReceive(RECEIVE_COMMAND * Struct)
  +
  + DESCRIPTION:    starts packet reception
  +
  + INPUT:			RECEIVE_COMMAND structure
  +
  + RETURN:         RF_OK: 				the operation was succesfull
  +                 RF_ERROR_STATE: 	the command is ignored, because the MAC 
  +										was not in STANDBY or IDLE state.
  +					RF_ERROR_TIMING: 	the command is ignored, there is
  +									 	not enough time to wake up
  +					RF_ERROR_PARAMETER: the command is ignored,some of the 
  +										input parameter(s) are
  +										out of the valid range
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFReceive(RECEIVE_COMMAND * Struct)
{
	uint16 temp16;
	uint8 temp8;

	if( RfState != sRFIdle )
		return RF_ERROR_STATE;

	//clear FIFO
	temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl2 << 8));
	temp8 |= 0x02;
	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl2), temp8);
	temp8 &= 0xFD;
	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl2), temp8);

	//set headers and filters
	if( Struct -> header.enabled_headers == 0 )
	{//header and header filter are disabled
		SpiWriteAddressData((REG_WRITE | HeaderControl1 ), 0x00);
	}
	else
	{//TODO: header and header filter are enabled - NOT YET IMPLEMENTED
	}

	//set preamble length (Rev X2)
	SpiWriteAddressData((REG_WRITE | PreambleLength), ((PREAMBLE_LENGTH << 1) + EXTRA_PREA));		

	//SpiWriteAddressData((REG_WRITE | ClockRecoveryGearshiftOverride), 0x52);
	//restart ADC
	SpiWriteAddressData((REG_WRITE | DeltasigmaADCTuning1), 0x80);
	SpiWriteAddressData((REG_WRITE | DeltasigmaADCTuning1), 0x00);

	//enable receiver chain
	temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
	temp8 |= 0x05;
	SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);	

	//these enable to see the RX signal on the spectrum analyzer
	#ifdef CHECK_RX_ON_SPA
	 SpiWriteAddressData((REG_WRITE | BlockEnableOverride2), 0xAC);
	 SpiWriteAddressData((REG_WRITE | BlockEnableOverride1), 0x10);
     SpiWriteAddressData((REG_WRITE | LDOControlOverride), 0x08);
	#endif

	//enable the wanted ITs
	SpiWriteAddressData((REG_WRITE | InterruptEnable1), 0x03);
	SpiWriteAddressData((REG_WRITE | InterruptEnable2), 0xC0);
	//release all IT flag
	ItStatus1 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus1 << 8));
	ItStatus2 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus2 << 8));

	//enable EXT IT
	EnableExt0It();
	//disable PHY timer
	StopTmr0();
	fPacketReceived = FALSE;

	//set next state
	RfState = sRFPreambleSearch;

	return RF_OK;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFGetStatus(PHY_STATUS * Struct)
  +
  + DESCRIPTION:    gives back the actual state of the PHY
  +
  + INPUT:			PHY_STATUS structure
  +
  + RETURN:         RF_OK: The operation was succesfull
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFGetStatus(PHY_STATUS * Struct)
{
	Struct -> status = RfState;
	return RF_OK;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFGetBuffer(RECEIVED_DATA * Struct)
  +
  + DESCRIPTION:    gives back the received packet and RSSI value. If packet was not
  +					recived it gives back only the RSSI value.
  +
  + INPUT:			RECEIVED_DATA structure
  +
  + RETURN:         RF_OK: operation was succesfull.
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
RF_ENUM RFGetBuffer(RECEIVED_DATA * Struct)
{	
	uint8 i;

	Struct -> fPacketReceived = fPacketReceived;
	//get header
	if( Struct -> message.header.enabled_headers != 0 )
	{	
		//TODO: implement this

	}
	//get received data and length
	Struct -> message.length =  (uint8)(0x00FF & SpiReadWriteWord(ReceivedPacketLength << 8)) + 1;
	for(i=0;i<Struct -> message.length;i++)
	{
		*Struct -> message.data++ = (uint8)(0x00FF & SpiReadWriteWord(FIFOAccess << 8));
	}
	return RF_OK;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  RF_ENUM RFGetRssiResult(uint8 * ant1, uint8 * ant2)
  +
  + DESCRIPTION:    gives back the RSSI values. If packet was not received
  +					it returns with RF_ERROR_PARAMETER
  +
  + INPUT:			None
  +
  + RETURN:         RF_OK: operation was succesfull.
  +					RF_ERROR_PARAMETER: packet was not received - the values are may not relevant
  +					ant1 - RSSI on ANT1 or RSSI register value (in case not using ant diversity)
  +					ant2 - RSSI on ANT2 (valid only if diersity is used)
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#pragma interrupt_level 1
RF_ENUM RFGetRssiResult(uint8 * ant1, uint8 * ant2)
{
	*ant1 = Rssi_ant1;
	*ant2 = Rssi_ant2;

	if( fPacketReceived == TRUE )
	{
		return RF_OK;
	}
	else
	{
		return RF_ERROR_PARAMETER;
	}
}

uint8 RFGetBatteryVoltage(void)
{
	return ((uint8)(0x00FF & SpiReadWriteWord(BatteryVoltageLevel << 8));
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void RFTimerISR(void)
  +
  + DESCRIPTION:    RF timer ISR routine
  +
  + INPUT:			None
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//#pragma interrupt_level 1
void RFTimerISR(void)
{
	#ifdef RF_LED_DEBUG
	LED2_PIN = 1;
	#endif

	StopTmr0();
	RFStateMachine( RF_TIMER_IT );
	DisableTmr0It();

	#ifdef RF_LED_DEBUG
	LED2_PIN = 0;
	#endif
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void RFExtISR(void)
  +
  + DESCRIPTION:    RF external ISR routine
  +
  + INPUT:			None
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
//#pragma interrupt_level 1
void RFExtISR(void)
{
	#ifdef RF_LED_DEBUG
	 LED3_PIN = 1;
	#endif

	ItStatus1 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus1 << 8));
	ItStatus2 = (uint8)(0x00FF & SpiReadWriteWord(InterruptStatus2 << 8));
	RFStateMachine( RF_EXT_IT );

	#ifdef RF_LED_DEBUG
	 LED3_PIN = 0;
	#endif
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void RFStateMachine(RF_ENUM ItSource)
  +
  + DESCRIPTION:    the state machine of the RF stack
  +
  +	INPUT:			ItSource - the interrupt source which called the state
  +							   machine (RF_TIMER_IT or RF_EXT_IT)
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void RFStateMachine(RF_ENUM ItSource)
{
uint16 temp16, timeout;
uint8 temp8;

	switch( RfState )
	{
		case sRFStandBy:
		break;
	
		case sRFIdle:
		break;
	
		case sRFWakingUp:
			StopTmr0();
			DisableExt0It();
			//check whether the chip woke up correctly or not
			if( ItSource == RF_EXT_IT )
			{//XTAL is running correctly
				RfState = sRFIdle;
			}
			else
			{//wake up error! XTAL is not running
				RfState = sRFWakeUpError;
			}
		break;
	
		case sRFWakeUpError:
		break;
	
		case sRFTxWaiting:
		break;
	
		case sRFTransmitting:
			temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
			temp8 &= 0xF1;
			SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);
			if( ItSource == RF_EXT_IT )
			{//EXT IT occured -> packet sent?
				if( (ItStatus1 & 0x04) == 0x04 )
				{//packet sent
					RfCbPacketSent();
					RfState = sRFPacketSent;
					StopTmr0();
					DisableExt0It();
				}
			}
			else
			{//TMR IT occured -> timeout, HW error
				StopTmr0();
				DisableExt0It();
				RfCbTxError();
				RfState = sRFTxError;
			}
			StopTmr0();
		break;
	
		case sRFTxError:
			StopTmr0();
		break;
	
		case sRFPacketSent:
			StopTmr0();
		break;
	
		case sRFRxWaiting:
		break;
	
		case sRFPreambleSearch:
			if( ItSource == RF_EXT_IT )
			{//EXT IT occured -> preamble detected?
				if( (ItStatus2 & 0x40) == 0x40 )
				{//preamble detected
					RfCbPreambleDetected();
					RfState = sRFSynchSearch;
					//start timeout for preamble detection
					StopTmr0();
					timeout = RX_PD_TIMEOUT * ByteTime;
					StartTmr0(TMR0_16BIT,BYTE_TIME_DIV,timeout,TRUE);
				}
			}
			else
			{//TMR interrupt 
			}
		break;
	
		case sRFSynchSearch:
			if( ItSource == RF_EXT_IT )
			{//EXT IT occured -> Synch word detected
				if(  (ItStatus2 & 0x80) == 0x80  )
				{//synch word detected
					//stop timer
					StopTmr0();
					//get RSSI value
					if( TestcardData.ant_type == ANT_DIVERSITY )
					{
						Rssi_ant2 = (uint8)(0x00FF & SpiReadWriteWord(AntennaDiversityRegister2 << 8));
						Rssi_ant1 =	(uint8)(0x00FF & SpiReadWriteWord(AntennaDiversityRegister1 << 8));
					}
					else
					{
						Rssi_ant1 = (uint8)(0x00FF & SpiReadWriteWord(ReceivedSignalStrengthIndicator << 8));
					}
					//call callback function
					RfCbSynchWordDetected();
					RfState = sRFReceiving;
					//start timeout for packet receiving
					timeout = ByteTime * RX_PACKET_LENGTH;
					StartTmr0(TMR0_16BIT,BYTE_TIME_DIV,timeout,TRUE);
				}
			}
			else
			{//TMR overflow -> false preamble detection
				StopTmr0();
				DisableExt0It();
				temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
				temp8 &= 0xF1;
				SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);
				RfCbFalsePreambleDetected();
				RfState = sRFFalsePreamble;
			}
		break;
	
		case sRFFalsePreamble:
		break;
	
		case sRFReceiving:
			temp8 = (uint8)(0x00FF & SpiReadWriteWord(OperatingFunctionControl1 << 8));
			temp8 &= 0xF1;
			SpiWriteAddressData((REG_WRITE | OperatingFunctionControl1), temp8);
			if( ItSource == RF_EXT_IT )
			{//EXT IT occured -> Packet valid
				//disable receiver
				if( (ItStatus1 & 0x02) == 0x02 )
				{//packet received
					RfCbPacketReceived();
					RfState = sRFPacketReceived;
					StopTmr0();
					DisableExt0It();
					fPacketReceived = TRUE;
				}
				if( (ItStatus1 & 0x01) == 0x01 )
				{//CRC Error
					RfCbCrcError();
					RfState = sRFRxError;
					StopTmr0();
					DisableExt0It();
				}
			}
			else
			{//TMR overflow -> HW error, reset everything
				StopTmr0();
				DisableExt0It();
				RfCbRxError();
				RfState = sRFRxError;
			}
		break;
	
		case sRFRxError:
			StopTmr0();
		break;
	
		case sRFPacketReceived:
		break;
	}
}

