/*
** ============================================================================
**
** FILE
**  $Id: IA443x_rf.h,v 1.0 2007/06/20 krk Exp apache $
**
** DESCRIPTION
**  Header file for the RF functions.
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#ifndef IA443X_RF_H
#define IA443X_RF_H

#include "PIC.h"
#define IA4431_REVX

                /* ======================================= *
                 *    APPLICATION SPECIFIC DEFINITIONS     *
                 * ======================================= */

//default settings 
#define PREAMBLE_LENGTH				(8)	//in bytes
#define SYNCH_WORD_LENGTH			(2)
#define PAYLOAD_LENGTH				(20)	//must be between 5...64!
#define LENGTH_FIELD				(1)
#define	MAX_PAYLOAD					(64)
#define CRC_LENGTH					(2)

#define PD_LENGTH					(4)		//in nibbles
#define PD_ERROR					(0)
#define EXTRA_PREA					(5)		//in nibbles


                /* ======================================= *
                 *          D E F I N I T I O N S          *
                 * ======================================= */
/*definitions for the GPIO Configuration 0,1,2 registers*/
//These functions can be multiplex out to the GPIO pins
#define	GPIO_NPOR					(0x00)	//only on GPIO0
#define	GPIO_POR					(0x00)	//only on GPIO1
#define	GPIO_UCLK					(0x00)	//only on GPIO2
#define	GPIO_WUT					(0x01)
#define	GPIO_LBD					(0x02)
#define	GPIO_DIN					(0x03)
#define	GPIO_IT_FALLING				(0x04)
#define	GPIO_IT_RISING				(0x05)
#define GPIO_IT_STATE				(0x06)
#define	GPIO_ADC					(0x07)
#define	GPIO_ATB_IN_N				(0x08)
#define	GPIO_ATB_IN_P				(0x09)
#define GPIO_DOUT					(0x0A)
#define	GPIO_DTB					(0x0B)
#define	GPIO_ATB_OUT_N				(0x0C)
#define	GPIO_ATB_OUT_P				(0x0D)
#define	GPIO_REF					(0x0E)
#define GPIO_TX_CLK					(0x0F)
#define GPIO_TX_DATA_IN				(0x10)
#define GPIO_EXT_RETX				(0x11)
#define GPIO_TX_STATE				(0xD2)
#define GPIO_TX_FIFO_AFULL			(0x13)
#define GPIO_RX_DATA				(0x14)
#define GPIO_RX_STATE				(0xD5)
#define GPIO_RX_FIFO_AFULL			(0x16)
#define GPIO_ANT1_SWITCH			(0xD7)
#define GPIO_ANT2_SWITCH			(0xD8)
#define GPIO_VALID_PREA				(0x19)
#define GPIO_INVALID_PREA			(0x1A)
#define GPIO_SYNCH_WORD_DET			(0x1B)
#define GPIO_CCA					(0x1C)
#define GPIO_VDD					(0x1D)
#define GPIO_GND					(0x1F)

//Register setting default values
#define	RF_MCU_LF_CLK				(0)
#define	RF_MCU_CLOCK_TAIL			(0)
#define	RF_MCU_CLK_RATE				(0)

//definitions for register usage
#define	REG_READ					(0x00)
#define	REG_WRITE					(0x80)

/*definitions for the Modulation Mode Control 2 register*/
//definitions for the TX CLK
#define	TX_CLK_OFF					(0x00)
#define	TX_CLK_ON_GPIO				(0x40)
#define	TX_CLK_ON_SCK				(0x80)
#define TX_CLK_ON_nIRQ				(0xC0)
//definitions for the Modulation source
#define	MOD_DIRECT_VIA_GPIO			(0x00)
#define MOD_DIRECT_VIA_SDI			(0x10)
#define MOD_FIFO					(0x20)
#define MOD_PN9						(0x30)
#define MOD_INVERT_TX				(0x08)
//definition for the modulation type
#define	MOD_TYPE_CW					(0x00)
#define MOD_TYPE_OOK				(0x01)
#define MOD_TYPE_FSK				(0x02)
#define MOD_TYPE_GFSK				(0x03)

/*definitions for the Data Access Control register*/
//definitions for the packet handler
#define TX_PACKET_HANDLER_DIS		(0x00)
#define TX_PACKET_HANDLER_EN		(0x08)
#define RX_PACKET_HANDLER_DIS		(0x00)
#define RX_PACKET_HANDLER_EN		(0x80)
#define LSB_FIRST					(0x40)
#define MSB_FIRST					(0x00)
#define CRC_DIS						(0x00)
#define CRC_CCITT					(0x04)
#define CRC_16						(0x05)
#define CRC_IEC16					(0x06)
#define CRC_BIACHEVA				(0x07)
#define CRC_DATA_ONLY				(0x20)
#define AUTO_PHASE_DETECT_MANCH		(0x10)

/*definitions for the TX Power register*/
#define RF_FULL_POWER				(0x07)

//time definitions @ 8MHz PIC
#define TX_PACKET_LENGTH			(PREAMBLE_LENGTH + SYNCH_WORD_LENGTH + LENGTH_FIELD + MAX_PAYLOAD + CRC_LENGTH + 5)
#define	RX_PD_TIMEOUT				(PREAMBLE_LENGTH + SYNCH_WORD_LENGTH + 4)
#define RX_PACKET_LENGTH			(LENGTH_FIELD + MAX_PAYLOAD + CRC_LENGTH + 3)
#define BYTE_TIME_DIV				(TMR0_32)
#define RESET_TIMEOUT				(1000)		//1ms			
#define RESET_TIMEOUT_DIV			(TMR0_2)
#define XTAL_WAKE_UP_PERIOD			(5000)		//5ms
#define XTAL_WAKE_UP_PERIOD_DEV		(TMR0_4)


                /* ======================================= *
                 *     T Y P E   D E C L A R A T I O N     *
                 * ======================================= */
//RF stack enumerations 
typedef enum _RF_ENUM
{
	RF_OK				=	0x00,			//function response parameters
	RF_ERROR_STATE		=	0x01,
	RF_ERROR_TIMING		=	0x02,
	RF_ERROR_PARAMETER	=	0x03,
	sRFStandBy			=	0x20,			//states
 	sRFPOR				=	0x22,
	sRFIdle				=	0x40,
	sRFWakingUp			=	0x21,
	sRFWakeUpError		=	0xA0,
	sRFTxWaiting		=	0x22,
	sRFTransmitting		=	0x64,
	sRFTxError			=	0xA4,
	sRFPacketSent		=	0x84,
	sRFRxWaiting		=	0x28,
	sRFPreambleSearch	=	0x68,
	sRFSynchSearch		=	0x69,
	sRFFalsePreamble	=	0xA8,
	sRFReceiving		=	0x6A,
	sRFRxError			=	0xA9,
	sRFPacketReceived	=	0x88,
	sRFScanWaiting		=	0x30,
	RF_TIMER_IT			=	0xB0,			//other enums
	RF_EXT_IT			=	0xB1,
} RF_ENUM;

typedef enum _RF_PARAMETER
{
	pRfFrequency,
	pRfGpios,
	pRfDigitalOut,
	pRfDigitalIn,
	pRfTransmitPower,
	pRfPreambleLength,
	pRfPremableDetection,
	pRfSynchronWord,
	pRfModulation,
	pRfPacketHandler,
	pRfAntennaDiversity,
	pLowBatteryDetector,
} RF_PARAMETER;

#define NMBR_OF_SAMPLE_SETTING		9
#define NMBR_OF_PARAMETER			12

typedef enum _RF_SAMPLE_SETTINGS
{
	DR2400BPS_DEV36KHZ			= 0,	/*DR = 2.4kbps;	Fdev = 36kHz; 	MOD index = 30;  	BW = 69.2kHz;  	Time = 209*/
	DR4800BPS_DEV45KHZ			= 1,	/*DR = 4.8kbps;	Fdev = 45kHz; 	MOD index = 18.8;  	BW = 90kHz;  	Time = 105*/
	DR9600BPS_DEV45KHZ			= 2,	/*DR = 9.6kbps; Fdev = 45kHz;   MOD index = 9.4;   	BW = 90kHz;  	Time = 53*/
	DR10KBPS_DEV5KHZ_MOD1		= 3,	/*DR = 10kbps;	Fdev = 5kHz; 	MOD index = 1; 		BW = 18.9kHz; 	Time = 50*/
	DR20KBPS_DEV10KHZ_MOD1		= 4,	/*DR = 20kbps;	Fdev = 10kHz; 	MOD index = 1; 		BW = 37.7kHz; 	Time = 25*/
	DR40KBPS_DEV20KHZ_MOD1		= 5,	/*DR = 40kbps;	Fdev = 20kHz; 	MOD index = 1; 		BW = 75.2kHz; 	Time = 13*/
	DR50KBPS_DEV25KHZ_MOD1		= 6,	/*DR = 50kbps;	Fdev = 25kHz; 	MOD index = 1; 		BW = 95.3kHz;  	Time = 10*/
	DR100KBPS_DEV50KHZ_MOD1		= 7,	/*DR = 100kbps;	Fdev = 50kHz; 	MOD index = 1; 		BW = 225.6kHz;  Time = 5*/
	DR125KBPS_DEV62KHZ5_MOD1	= 8,	/*DR = 125kbps;	Fdev = 62.5kHz; MOD index = 1; 		BW = 249.6kHz; 	Time = 4*/
} RF_SAMPLE_SETTINGS;

typedef struct _HEADER
{
	uint8		header3;
	uint8		header2;
	uint8		header1;
	uint8		header0;
	uint8		enabled_headers;
} HEADER;

typedef struct _HEADER_FLT
{
	uint8		all_filter_disable	:1;
	uint8		header3_flt			:1;
	uint8		header2_flt			:1;
	uint8		header1_flt			:1;
	uint8		header0_flt			:1;
	uint8		header3_bc_flt		:1;
	uint8		header2_bc_flt		:1;
	uint8		header1_bc_flt		:1;
	uint8		header0_bc_flt		:1;
	uint8		packet_length_flt	:1;
	uint8		reserve_bits		:6;
} HEADER_FLT;

typedef	struct	_MESSAGE
{
	HEADER		header;
	uint8		length;
	uint8	*	data;
} MESSAGE;

typedef struct _RECEIVE_COMMAND
{
	HEADER		header;
	HEADER_FLT	header_flt;
} RECEIVE_COMMAND;

typedef struct _PHY_STATUS
{
	uint8		status;
} PHY_STATUS;

typedef struct _RECEIVED_DATA
{
	uint8		fPacketReceived;
	MESSAGE		message;
} RECEIVED_DATA;

typedef struct _REVX_CONF
{
	uint8	chip_version;				//0x01: revX non using frequency offset; 0x02: revX using frequency offset
	uint8	board_number;				//0 ... 255 used for defining the SID
	uint8	vco_current_trimming;		//VCO current trimming register setting
	uint8	xtal_cap_bank;				//crystal load cap register setting
	uint8 	frequency_band;				//0xff: high band (868 - 950MHz), 0xfe: 434MHz
	uint8	antenna_connection;			//0xff: sep. RX&TX SMA, 0xfe: TRX switch (GPIO1&2), 0xfd: diversity (GPIO1&2)
} REVX_CONF;

typedef union _RF_CONFIG
{
	REVX_CONF	name;
	uint8		array[6];
} RF_CONFIG;

typedef enum _RF_REG_MAP					//revX
{
  DeviceType 								= 0x00,
  DeviceVersion								= 0x01,
  DeviceStatus 								= 0x02,
  InterruptStatus1 							= 0x03,
  InterruptStatus2 							= 0x04,
  InterruptEnable1 							= 0x05,          
  InterruptEnable2 							= 0x06,         
  OperatingFunctionControl1 				= 0x07,
  OperatingFunctionControl2 				= 0x08,
  CrystalOscillatorLoadCapacitance 			= 0x09,
  MicrocontrollerOutputClock 				= 0x0A,
  GPIO0Configuration 						= 0x0B,
  GPIO1Configuration 						= 0x0C,         
  GPIO2Configuration						= 0x0D,
  IOPortConfiguration						= 0x0E,
  ADCConfiguration							= 0x0F,
  ADCSensorAmplifierOffset					= 0x10,
  ADCValue									= 0x11,
  TemperatureSensorControl					= 0x12,
  TemperatureValueOffset					= 0x13,
  WakeUpTimerPeriod1 						= 0x14,          
  WakeUpTimerPeriod2 						= 0x15,         
  WakeUpTimerPeriod3 						= 0x16,         
  WakeUpTimerValue1							= 0x17,
  WakeUpTimerValue2							= 0x18,
  LowDutyCycleModeDuration 					= 0x19,       
  LowBatteryDetectorThreshold  				= 0x1A,
  BatteryVoltageLevel 						= 0x1B,                          
  IFFilterBandwidth  						= 0x1C,                           
  AFCLoopGearshiftOverride					= 0x1D,
  AFCTimingControl 							= 0x1E,                              
  ClockRecoveryGearshiftOverride 			= 0x1F,              
  ClockRecoveryOversamplingRatio 			= 0x20,              
  ClockRecoveryOffset2 						= 0x21,                       
  ClockRecoveryOffset1 						= 0x22,                       
  ClockRecoveryOffset0 						= 0x23,                     
  ClockRecoveryTimingLoopGain1 				= 0x24,              
  ClockRecoveryTimingLoopGain0 				= 0x25,             
  ReceivedSignalStrengthIndicator 			= 0x26,          
  RSSIThresholdForClearChannelIndicator 	= 0x27,   
  AntennaDiversityRegister1					= 0x29,
  AntennaDiversityRegister2					= 0x2A,
  DataAccessControl 						= 0x30,                          
  EZmacStatus 								= 0x31,                                  
  HeaderControl1 							= 0x32,                               
  HeaderControl2 							= 0x33,                              
  PreambleLength 							= 0x34,                               
  PreambleDetectionControl 					= 0x35,                    
  SyncWord3 								= 0x36,                                   
  SyncWord2 								= 0x37,                                   
  SyncWord1 								= 0x38,                               
  SyncWord0 								= 0x39,                                
  TransmitHeader3							= 0x3A,                       
  TransmitHeader2 							= 0x3B,                             
  TransmitHeader1 							= 0x3C,                              
  TransmitHeader0 							= 0x3D,                             
  TransmitPacketLength 						= 0x3E,                         
  CheckHeader3 								= 0x3F,                                
  CheckHeader2 								= 0x40,                              
  CheckHeader1 								= 0x41,                             
  CheckHeader0 								= 0x42,                            
  HeaderEnable3 							= 0x43,                               
  HeaderEnable2 							= 0x44,                                 
  HeaderEnable1 							= 0x45,                                
  HeaderEnable0 							= 0x46,                              
  ReceivedHeader3 							= 0x47,                          
  ReceivedHeader2 							= 0x48,                         
  ReceivedHeader1 							= 0x49,                           
  ReceivedHeader0 							= 0x4A,                             
  ReceivedPacketLength						= 0x4B,
  AnalogTestBus 							= 0x50,                              
  DigitalTestBus 							= 0x51,                          
  TXRampControl 							= 0x52,                             
  PLLTuneTime 								= 0x53,                            
  CalibrationControl 						= 0x55,                     
  ModemTest 								= 0x56,                               
  ChargepumpTest 							= 0x57,                    
  ChargepumpCurrentTrimming_Override 		= 0x58,         
  DividerCurrentTrimming				 	= 0x59,    
  VCOCurrentTrimming 						= 0x5A,                           
  VCOCalibration_Override 					= 0x5B,                    
  SynthesizerTest 							= 0x5C,                              
  BlockEnableOverride1 						= 0x5D,                        
  BlockEnableOverride2 						= 0x5E,                      
  BlockEnableOverride3 						= 0x5F,                       
  ChannelFilterCoefficientAddress 			= 0x60,             
  ChannelFilterCoefficientValue 			= 0x61,            
  CrystalOscillator_ControlTest 			= 0x62,               
  RCOscillatorCoarseCalibration_Override 	= 0x63,    
  RCOscillatorFineCalibration_Override 		= 0x64,      
  LDOControlOverride 						= 0x65,                          
  DeltasigmaADCTuning1			 			= 0x67,
  DeltasigmaADCTuning2			 			= 0x68,
  AGCOverride1					 			= 0x69,
  AGCOverride2 								= 0x6A,
  GFSKFIRFilterCoefficientAddress 			= 0x6B,            
  GFSKFIRFilterCoefficientValue 			= 0x6C,              
  TXPower 									= 0x6D,                                   
  TXDataRate1 								= 0x6E,                            
  TXDataRate0 								= 0x6F,                              
  ModulationModeControl1 					= 0x70,                   
  ModulationModeControl2 					= 0x71,                   
  FrequencyDeviation 						= 0x72,                            
  FrequencyOffset 							= 0x73,                            
  FrequencyChannelControl					= 0x74,
  FrequencyBandSelect 						= 0x75,                        
  NominalCarrierFrequency1	 				= 0x76,                    
  NominalCarrierFrequency0 					= 0x77,                    
  FrequencyHoppingChannelSelect 			= 0x79,               
  FrequencyHoppingStepSize 					= 0x7A,                     
  TXFIFOControl1 							= 0x7C,                        
  TXFIFOControl2 							= 0x7D,    
  RXFIFOControl 							= 0x7E,                               
  FIFOAccess								= 0x7F,
} RF_REG_MAP;

                /* ======================================= *
                 *  F U N C T I O N   P R O T O T Y P E S  *
                 * ======================================= */
RF_ENUM RfInitHw(void);
RF_ENUM RFWakeUp(void);
RF_ENUM RFSleep(void);
RF_ENUM RFIdle(void);
RF_ENUM RFSetParameter(RF_PARAMETER parameter, uint8 * value, uint8 length);
RF_ENUM RFSetRfParameters(RF_SAMPLE_SETTINGS setting);
RF_ENUM RFDirectTransmit(void);
RF_ENUM RFTransmit(MESSAGE * Struct);
RF_ENUM RFDirectReceive(void);
RF_ENUM RFReceive(RECEIVE_COMMAND * Struct);
RF_ENUM RFGetStatus(PHY_STATUS * Struct);
RF_ENUM RFGetBuffer(RECEIVED_DATA * Struct);
RF_ENUM RFGetRssiResult(uint8 * ant1, uint8 * ant2);
void RFTimerISR(void);
void RFExtISR(void);

#endif
