/*
** ============================================================================
**
** FILE
**  $Id: IA443x_test.c,v 1.0 2007/12/10 krk Exp apache $
**
** DESCRIPTION
**  Contains the radio test functions
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "uart.h"
#include "pic18_timers.h"
#include "dog_glcd.h"
#include "IA443x_rf.h"
#include "IA443x_demo.h"
#include "IA443x_test.h"
#include "menu.h"
#include "global_definitions.h"


#ifdef TEST_FUNCTIONS_ENABLED
/*------------------------------------------------------------------------*/
/*						GLOBAL variables								  */
/*------------------------------------------------------------------------*/
extern volatile S_TEST sDemo;
extern volatile DEMO_STATES DemoStates;
extern uint8 rf_param[10];
extern uint16 NmbrOfSentPackets, NmbrOfReceivedPackets;

extern PHY_STATUS RfStatus;
extern uint8 RxTxBuffer[MAX_LENGTH_OF_PAYLOAD];
extern HEADER HeaderBuffer;
extern MESSAGE MessageBuffer;
extern RECEIVE_COMMAND RxCommand;
extern RECEIVED_DATA ReceivedData;

extern DEMO_PARAMETERS demo_param;

volatile uint8 rf_setting;
uint8 string[4];
uint8 lcd_data[22];
/*------------------------------------------------------------------------*/
/*						LOCAL function prototypes						  */
/*------------------------------------------------------------------------*/


/*------------------------------------------------------------------------*/
/*						LOCAL definitions								  */
/*------------------------------------------------------------------------*/
#define PER_RESET_INPUT	PB2_PIN


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void TestInit(void)
  +
  + DESCRIPTION:    initialize the test state machines
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void TestInit(void)
{
	//initialize the RF stack
	RfInitHw();
	//initialize the test variables
	DemoStates = sDemoInit;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void TestPerStateMachine(void)
  +
  + DESCRIPTION:    state machine, which realize the PER test
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void TestPerStateMachine(void)
{
uint8 temp8;
uint16 delay;
float temp_float;

	switch( DemoStates )
	{
		case sDemoInit:
			//draw start screen
			PerformMenu( sMenuPERStartScreen );

			/**********************************/
			/*initialize RF stack for the test*/
			/**********************************/
			SetRadio( rf_setting );

			//GPIO0 - RX_CLK
		 	SpiWriteAddressData((REG_WRITE | GPIO0Configuration), GPIO_RX_DATA );	

			//clear variables
			NmbrOfSentPackets = 0;
			NmbrOfReceivedPackets = 0;

			//next state
			DemoStates = sDemoStartReceive;	
		break;

		case sDemoStartReceive:
			//reset the RF stack
			RFIdle();
			//set variables
			if( (NmbrOfSentPackets == 65535) || (PER_RESET_INPUT == 0) )
			{
				while( PER_RESET_INPUT == 0 );
				NmbrOfSentPackets = 1;
				NmbrOfReceivedPackets = 0;
			}
			else
			{
				NmbrOfSentPackets++;

			}
			//start receive
			RFReceive(&RxCommand);
			//some delay
			StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, 5000, FALSE);
			while(Tmr3Expired() == FALSE);			
			//generate trigger
			LED1_PIN = 1;
			//start timeout 200ms
			StartTmr3(TMR3_INTERNAL_ENABLE, TMR3_8, 50000, FALSE);
			//release trigger
			LED1_PIN = 0;
			//next state
			DemoStates = sDemoWaitForReceive;
		break;

		case sDemoWaitForReceive:
			if( Tmr3Expired() == TRUE )
			{//packet didn't receive within timeout
				//stop timer & RF Stack
				StopTmr3();
				RFIdle();
				DemoStates = sDemoPrintResult;
			}
		break;

		case sDemoPacketReceived:
			//stop timer
			StopTmr3();
			//get received packet and put it into the buffer 
			MessageBuffer.data = &RxTxBuffer[10];
			ReceivedData.message = MessageBuffer;
			RFGetBuffer(&ReceivedData);
			//stop RF stack
			RFIdle();
			//inc counter
			NmbrOfReceivedPackets++;
			DemoStates = sDemoPrintResult;
		break;


		case sDemoRxError:
			//stop RF stack and timer
			RFIdle();
			StopTmr3();
			DemoStates = sDemoPrintResult;
		break;

		case sDemoPrintResult:
			//calc PER
			temp_float = CalcPer(NmbrOfSentPackets,NmbrOfReceivedPackets);
			MenuReportTestPer(NmbrOfSentPackets, NmbrOfReceivedPackets, temp_float);
			//next state
			DemoStates = sDemoStartReceive;
		break;

		default:
		break;
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void TestBerStateMachine(void)
  +
  + DESCRIPTION:    state machine, which realize the BER test
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void TestBerStateMachine(void)
{	
uint8 temp8;
uint16 temp16;

	switch( DemoStates )
	{
		case sDemoInit:
			PerformMenu( sMenuBERStartScreen );							
			/**********************************/
			/*initialize RF stack for the demo*/
			/**********************************/
			SetRadio( rf_setting );

			//GPIO0 - RX_DATA
		 	SpiWriteAddressData((REG_WRITE | GPIO0Configuration), GPIO_RX_DATA );	

			//SET direct modulation mode
			rf_param[0] = TX_CLK_ON_GPIO | MOD_DIRECT_VIA_GPIO | MOD_TYPE_GFSK;
			RFSetParameter(pRfModulation, rf_param, 1);
			//disable packet handler
			rf_param[0] = RX_PACKET_HANDLER_DIS | CRC_DIS;
			RFSetParameter(pRfPacketHandler, rf_param, 1);
			//start receiving
			RFDirectReceive();
			//next state
			DemoStates = sDemoWaitForReceive;
		break;

		case sDemoWaitForReceive:
		break;

		default:
		break;
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void TestCwStateMachine(void)
  +
  + DESCRIPTION:    state machine, which realize the CW test
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void TestCwStateMachine(uint8 cw_pn9)
{
uint8 temp8; 

	switch( DemoStates )
	{
		case sDemoInit:
			/**********************************/
			/*initialize RF stack for the demo*/
			/**********************************/
			SetRadio( rf_setting );

			//SET direct modulation mode
			if( cw_pn9 == MOD_TYPE_CW )
			{
				PerformMenu( sMenuCWStartScreen );
				rf_param[0] = TX_CLK_ON_GPIO | MOD_TYPE_CW;
			}
			else
			{
				PerformMenu( sMenuPN9StartScreen );
				rf_param[0] = TX_CLK_ON_GPIO | MOD_TYPE_GFSK | MOD_PN9;
			}
			RFSetParameter(pRfModulation, rf_param, 1);
			//disable packet handler
			rf_param[0] = TX_PACKET_HANDLER_DIS | CRC_DIS;
			RFSetParameter(pRfPacketHandler,rf_param,1);	

			//start CW mode
			RFDirectTransmit();
			//next state
			DemoStates = sDemoSendPing;
		break;

		case sDemoSendPing:
		default:
		break;
	}
}

#endif
