/*
** ============================================================================
**
** FILE
**  $Id: IAI_EBID.c,v 1.0 2007/09/06 krk Exp apache $
**
** DESCRIPTION
**  Testcard Board Identification EEPROM handling
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#include "IAI_EBID.h"
#include <stdio.h>
#include <string.h>
#include "IA443x_rf.h"

/*------------------------------------------------------------------------*/
/*						GLOBAL variables								  */
/*------------------------------------------------------------------------*/
EBID_DATA	TestcardData;
SPECIAL_REGISTERS SpecialRegisters;

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  
  +
  + DESCRIPTION:    
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
EBID_DATABASES GetEBIDDataFromEEPROM(void)
{
uint8 temp8, i, string[9];
uint16 temp16;

LED2_PIN = 1;
LED2_PIN = 0;

	TestcardData.valid_info = FALSE;

	//figure out what database (s) are available
	temp8 = ReadByteFromTestcardEEPROM( 0x0000 );
	if( (temp8 & POWER_DB) == POWER_DB )
	{//the Power Description Database is implemented, use this
		return POWER_DB;
	}
	if( (temp8 & WIRELINE_DB) == WIRELINE_DB )
	{//the Wireline Description Database is implemented, use this
		return WIRELINE_DB;
	}
	if( (temp8 & WIRELESS_DB) == WIRELESS_DB )
	{//the Wireless Description Database is implemented, use this
		TestcardData.valid_info = TRUE;
		//find out the chip type
		TestcardData.chip_type = NOT_AVAILABLE;
		ReadSegmentFromTestcardEEPROM(0x0010, string, 8);
		if( memcmp(&string[0], "IA4220", 6) == 0 )	TestcardData.chip_type = IA4220;
		if( memcmp(&string[0], "IA4221", 6) == 0 )	TestcardData.chip_type = IA4221;
		if( memcmp(&string[0], "IA4222", 6) == 0 )	TestcardData.chip_type = IA4222;
		if( memcmp(&string[0], "IA4230", 6) == 0 )	TestcardData.chip_type = IA4230;
		if( memcmp(&string[0], "IA4231", 6) == 0 )	TestcardData.chip_type = IA4231;
		if( memcmp(&string[0], "IA4315", 6) == 0 )	TestcardData.chip_type = IA4315;
		if( memcmp(&string[0], "IA4320", 6) == 0 )	TestcardData.chip_type = IA4320;
		if( memcmp(&string[0], "IA4322", 6) == 0 )	TestcardData.chip_type = IA4322;
		if( memcmp(&string[0], "IA4330", 6) == 0 )	TestcardData.chip_type = IA4330;
		if( memcmp(&string[0], "IA4322", 6) == 0 )	TestcardData.chip_type = IA4322;
		if( memcmp(&string[0], "IA4420", 6) == 0 )	TestcardData.chip_type = IA4420;
		if( memcmp(&string[0], "IA4421", 6) == 0 )	TestcardData.chip_type = IA4421;
		if( memcmp(&string[0], "IA4430", 6) == 0 )	TestcardData.chip_type = IA4430;
		if( memcmp(&string[0], "IA4431", 6) == 0 )	TestcardData.chip_type = IA4431;
		if( memcmp(&string[0], "IA4432", 6) == 0 )	TestcardData.chip_type = IA4432;		
		if( memcmp(&string[0], "IA4520", 6) == 0 )	TestcardData.chip_type = IA4520;
		//get chip revision
		ReadSegmentFromTestcardEEPROM(0x0018, &TestcardData.chip_revision[0], 2);
		//get serial number
		temp8 = ReadByteFromTestcardEEPROM( 0x0034 );
		TestcardData.serial_nmbr = ((uint16)temp8 << 8) + ReadByteFromTestcardEEPROM( 0x0035 );
		//get supported frequency bands
		temp8 = ReadByteFromTestcardEEPROM( 0x0043 );
		TestcardData.freq_band = ((uint16)temp8 << 8) + ReadByteFromTestcardEEPROM( 0x0044 );
		//get antenna connection type
		TestcardData.ant_type = ReadByteFromTestcardEEPROM( 0x0045 );
		//get special registers
		temp8 = ReadByteFromTestcardEEPROM( 0x004F);
		SpecialRegisters.nmbr_of_registers = temp8;
		if( temp8 > 0 )
		{
			for(i=0;i<temp8;i++)
			{
				temp16 = ((uint16)ReadByteFromTestcardEEPROM(0x0050 + (i << 1) )) << 8;
				temp16 += (uint16)ReadByteFromTestcardEEPROM(0x0050 + ((i << 1)+1));
				SpecialRegisters.registers[i] = temp16;
			}
		}
		return WIRELESS_DB;
	}
	//check whether demo specific database is implemented or not
	switch( temp8 & 0x0F )
	{
		case EZRADIO_PRO:
			TestcardData.valid_info = TRUE;
			TestcardData.chip_type = IA4432;
			TestcardData.chip_revision[0] = 'X';
			TestcardData.chip_revision[1] = '2';
			TestcardData.serial_nmbr = (uint16)ReadByteFromTestcardEEPROM( 0x0001 );
			TestcardData.freq_band = 0x0300;			//868-960MHz
			TestcardData.ant_type = SEPARATE_SMA;
			//set special registers
			SpecialRegisters.nmbr_of_registers = 2;
			SpecialRegisters.registers[0] = ((uint16)(VCOCurrentTrimming | 0x80) << 8) + (uint16)ReadByteFromTestcardEEPROM( 0x0002 );
			SpecialRegisters.registers[1] = ((uint16)( CrystalOscillatorLoadCapacitance | 0x80) << 8) + (uint16)ReadByteFromTestcardEEPROM( 0x0003 );
		break;

		case EZRADIO_PRO_OFFSET:
			TestcardData.valid_info = TRUE;
			TestcardData.chip_type = IA4432;
			TestcardData.chip_revision[0] = 'X';
			TestcardData.chip_revision[1] = '2';
			TestcardData.serial_nmbr = (uint16)ReadByteFromTestcardEEPROM( 0x0001 );
			//figure out the frequency band
			temp8 = ReadByteFromTestcardEEPROM( 0x0004 );
			switch( temp8 )
			{
				case 0xFE:
					TestcardData.freq_band = 0x0010;			//434MHz
				break;

				case 0xFF:
				default:
					TestcardData.freq_band = 0x0300;			//868-960MHz
				break;
			}
			//figure out the antenna connection type
			temp8 = ReadByteFromTestcardEEPROM( 0x0005 );
			switch( temp8 )
			{
				case 0xFD:
					TestcardData.ant_type = ANT_DIVERSITY;			
				break;

				case 0xFE:
					TestcardData.ant_type = SMA_TRX_SWITCH;			
				break;

				case 0xFF:
				default:
					TestcardData.ant_type = SEPARATE_SMA;			
				break;
			}
			//set special registers
			SpecialRegisters.nmbr_of_registers = 4;
			SpecialRegisters.registers[0] = ((uint16)(VCOCurrentTrimming | 0x80) << 8) + (uint16)ReadByteFromTestcardEEPROM( 0x0002 );
			SpecialRegisters.registers[1] = ((uint16)(CrystalOscillatorLoadCapacitance | 0x80) << 8) + (uint16)ReadByteFromTestcardEEPROM( 0x0003 );
			SpecialRegisters.registers[2] = ((uint16)(0x80 | FrequencyOffset) << 8) + 0x00E3;
			SpecialRegisters.registers[3] = ((uint16)(0x80 | FrequencyChannelControl) << 8) + 0x0003;
		break;

		default:
			//no special database is available
			return NO_EEPROM;
		break;
	}
	return NO_EEPROM;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void SetSpecialRegisters(void)
  +
  + DESCRIPTION:    set all the special (not default) registers of the chip
  +	
  + RETURN:         None
  +
  +	INPUT:			None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void SetSpecialRegisters(void)
{
uint8 i;
	for(i=0;i<SpecialRegisters.nmbr_of_registers;i++)
	{
		SpiWriteWord(SpecialRegisters.registers[i]);
	}		
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 ReadByteFromTestcardEEPROM(uint16 address)
  +
  + DESCRIPTION:    read on byte from the EEPROM populated to the Testcard
  +	
  + RETURN:         value read out
  +
  +	INPUT:			address of the data in the EEPROM
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 ReadByteFromTestcardEEPROM(uint16 address)
{
uint8 temp8;

	//select the EEPROM
	EE_NSEL_PIN = 0;
	//send instruction + address MSB
	SpiWrite(0x03);
	//send address upper byte
	SpiWrite((uint8)(address >> 8));
	//send address lower byte
	SpiWrite((uint8)(address & 0x00FF));
	//get data
	temp8 = SpiReadWrite(0x00);
	//release nSEL
	EE_NSEL_PIN = 1;

	return temp8;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void ReadSegmentFromTestcardEEPROM(uint16 star_address, uint8 * data, uint8 length)
  +
  + DESCRIPTION:    read a segment from the EEPROM populated to the Testcard
  +	
  + RETURN:         data stored in the place where the data pointer defined
  +
  +	INPUT:			start_address - starting address of the segment needs to be read out
  +					length - length of the segment needs to be read out
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void ReadSegmentFromTestcardEEPROM(uint16 start_address, uint8 * data, uint8 length)
{
uint8 temp8;

	//select the EEPROM
	EE_NSEL_PIN = 0;
	//send instruction + address MSB
	SpiWrite(0x03);
	//send address upper byte
	SpiWrite((uint8)(start_address >> 8));
	//send address lower byte
	SpiWrite((uint8)(start_address & 0x00FF));
	for(temp8=0;temp8<length;temp8++)
	{
		//get data
		*data++ = SpiReadWrite(0x00);
	}
	//release nSEL
	EE_NSEL_PIN = 1;
}
