/*
** ============================================================================
**
** FILE
**  $Id: IAI_EBID.h,v 1.0 2007/08/28 krk Exp apache $
**
** DESCRIPTION
**  Header file for the Testcard Board Identification EEPROM usage
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#ifndef IAI_EBID_H
#define IAI_EBID_H

#include "PIC.h"

                /* ======================================= *
                 *          D E F I N I T I O N S          *
                 * ======================================= */
typedef enum _EBID_DATABASES
{
	NO_EEPROM			= 0, 
	EZRADIO_PRO			= 1,
	EZRADIO_PRO_OFFSET,
	POWER_DB			= 0x10,
	WIRELINE_DB			= 0x20,
	WIRELESS_DB			= 0x40,
} EBID_DATABASES;

typedef enum _CHIP_TYPE
{
	IA4220,	
	IA4221,
	IA4222,
	IA4230,
	IA4231,
	IA4315,
	IA4320,
	IA4322,
	IA4330,
	IA4420,
	IA4421,
	IA4430,
	IA4431,
	IA4432,
	IA4520,
} CHIP_TYPE;

typedef enum _ANT_TYPE
{
	SMA,
	SMA_TRX_SWITCH,
	SEPARATE_SMA,
	ANT_DIVERSITY,
	PCB_ANT,
} ANT_TYPE;

#define NOT_AVAILABLE		0xFF

typedef struct _EBID_DATA
{
	uint8		valid_info;
	CHIP_TYPE	chip_type;
	uint8		chip_revision[2];
	uint16		serial_nmbr;
	uint16		freq_band;
	ANT_TYPE	ant_type;
} EBID_DATA;

typedef struct _SPECIAL_REGISTERS
{
	uint8	nmbr_of_registers;
	uint16	registers[16];
} SPECIAL_REGISTERS;

                /* ======================================= *
                 *            V A R I A B L E S            *
                 * ======================================= */

                /* ======================================= *
                 *  F U N C T I O N   P R O T O T Y P E S  *
                 * ======================================= */
EBID_DATABASES GetEBIDDataFromEEPROM(void);
void SetSpecialRegisters(void);
uint8 ReadByteFromTestcardEEPROM(uint16 address);
void ReadSegmentFromTestcardEEPROM(uint16 start_address, uint8 * data, uint8 length);

#endif
