/*
** ============================================================================
**
** FILE
**  $Id: ARA_Test_PIC.c,v 1.0 2007/06/20 krk Exp apache $
**
** DESCRIPTION
**  Contains all the low level, PIC dependent functions		    
**	This function has to be included as a C file into the main source file!		
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#ifndef PIC_C
#define PIC_C

#include "PIC.h"
#include "pic18_timers.h"
#include "uart.h"
#include "IA443x_rf.h"
#include "IA443x_demo.h"

/*------------------------------------------------------------------------*/
/*						CONFIGURATION BITS								  */
/*------------------------------------------------------------------------*/
__CONFIG(1, IESODIS & FCMDIS & INTIO);
__CONFIG(2, BOREN & PWRTEN & WDTDIS);
__CONFIG(3, MCLREN & LPT1DIS & PBDIGITAL & CCP2RB3);
__CONFIG(4, XINSTDIS & DEBUGDIS & LVPDIS & STVRDIS);
__CONFIG(5, UNPROTECT);

/*------------------------------------------------------------------------*/
/*						DEFAULT EEPROM CONTENT							  */
/*------------------------------------------------------------------------*/
__EEPROM_DATA(' ',' ',' ',' ',' ',' ',' ',' '); //defaut setting of the customer name
__EEPROM_DATA(' ',' ',' ',' ',' ',' ',' ',' '); //defaut setting of the customer name
__EEPROM_DATA(' ',' ',' ',' ',' ',' ',' ',' '); //it accept only up to CUSTOMER_NAME_SIZE of characters	
__EEPROM_DATA(0,0,0,0,0,0,0,0);					//first_plug,def_br,def_freq,did

/*------------------------------------------------------------------------*/
/*						INTERRUPT HANDLING								  */
/*------------------------------------------------------------------------*/
#pragma interrupt_level 1
void interrupt isr_routine(void)
{
	/*TIMER INTERRUPTS*/
	if( TMR0IF == 1 && TMR0IE == 1 )
	{
		TMR0IF = 0;
		RFTimerISR();
	}
	if( TMR1IF == 1 && TMR1IE == 1 )
	{
		TMR1IF = 0;
		IncrementSystemTimer();
	}
	if( TMR2IF == 1 && TMR2IE == 1 )
	{
		TMR2IF = 0;
	}
	if( TMR3IF == 1 && TMR3IE == 1 )
	{
		TMR3IF = 0;
	}
	/*EXTERNAL INTERRUPTS*/
	if( INT0IF == 1 && INT0IE == 1 )
	{
		INT0IF = 0;
		RFExtISR();
	}		
	if( INT1IF == 1 && INT1IE == 1 )
	{
		INT1IF = 0;
	}
	if( INT2IF == 1 && INT2IE == 1 )
	{
		INT2IF = 0;
	}
	/*UART RX INTERRUPTS*/
	if( RCIF == 1 && RCIE == 1 )
	{//NOTE: flag is cleared automatically after reading the data from the buffer		
		UartRxISR();	
	}
	/*PUSH BUTTON INTERRUPTS*/
	if( RBIF == 1 && RBIE == 1 )
	{
		RBIF = 0;
	}
}

/*------------------------------------------------------------------------*/
/*			LOW LEVEL PIC PERIPHERAL HANDLING FUNCTIONS		  			  */
/*------------------------------------------------------------------------*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void SetHwMasterSpi(void)
  +
  + DESCRIPTION:    initialize the hardware SPI peripheral of the PIC to
  +					operate as a Master SPI port
  +
  +	INPUT:			None
  +
  + RETURN:         None
  +
  + NOTES:          has to be called in the power-on routine
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void SetHwMasterSpi(void)
{
	//set I/O port directions
	SCK_DIR 		= OUTPUT;
	MISO_DIR	 	= INPUT;
	MOSI_DIR	 	= OUTPUT;

	MOSI_PIN	 	= 1;
	SCK_PIN 		= 1;


	//SDO is set at the falling edge of the SCK
	//SDI sample is taken at the rising edge of the SCK
	SSPCON1 = ENABLE_MSSP | SCK_LOW_DURING_IDLE | SPI_MASTER_FOSC4;
	SSPSTAT = 0x40;
	//disable SPI interrupt, the software polls the SPI IT flag
	SSPIE = 0;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void SpiWrite(uint8 data)
  +
  + DESCRIPTION:    sends 8 length data through the SPI port
  +
  +	INPUT:			data 
  +
  + RETURN:         None
  +
  + NOTES:          It doesn't control the nSEL pin
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void SpiWrite(uint8 data)
{
	DisableGlobalIt();
	#ifdef HW_SPI
	 SSPBUF = data;
	 #asm
				nop	
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
	 #endasm
	 SSPIF = 0;
	#else
	#endif
	EnableGlobalIt();
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 SpiReadWrite(uint8 data)
  +
  + DESCRIPTION:    sends and read 8 length data through the SPI port
  +
  +	INPUT:			data 
  +
  + RETURN:         received byte
  +
  + NOTES:          it doesn't control the nSEL pin
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 SpiReadWrite(uint8 data)
{
	#ifdef HW_SPI
	 DisableGlobalIt();
 	 SSPBUF = data;						
	 #asm
		spirw:	btfss 	c:_SSPSTAT,0
				bra 	spirw	
	 #endasm
 	 EnableGlobalIt();
	 return SSPBUF;
	#else
	#endif
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void SpiWriteByte(uint8 data)
  +
  + DESCRIPTION:    sends 8 length data through the SPI port
  +
  +	INPUT:			data 
  +
  + RETURN:         None
  +
  + NOTES:          It controls the nSEL pin
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void SpiWriteByte(uint8 data)
{
	DisableGlobalIt();
	RF_NSEL_PIN = 0; 
	#ifdef HW_SPI
 	 SSPBUF = data;						
	 #asm
				nop	
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
	 #endasm
	#else
	#endif 
	RF_NSEL_PIN = 1;
	EnableGlobalIt();
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void SpiWriteWord(uint16 data)
  +
  + DESCRIPTION:    sends 16 length data through the SPI port
  +
  +	INPUT:			data 
  +
  + RETURN:         None
  +
  + NOTES:          It controls the nSEL pin
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void SpiWriteWord(uint16 data)
{
	DisableGlobalIt();
	RF_NSEL_PIN = 0; 
	#ifdef HW_SPI
 	 SSPBUF = (uint8)(data >> 8);						
 	 #asm
				nop	
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
	 #endasm
 	 SSPBUF = (uint8)(data & 0x00FF);						
	 #asm
				nop	
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
	 #endasm
	#else
	#endif
	RF_NSEL_PIN = 1;
	EnableGlobalIt();
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void SpiWriteAddressData(uint8 address, uint8 data)
  +
  + DESCRIPTION:    sends 16 length data through the SPI port (address and data)
  +
  +	INPUT:			address - register address
  +					data 	- 8bit data 
  +
  + RETURN:         None
  +
  + NOTES:          It controls the nSEL pin
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#pragma interrupt_level 1
void SpiWriteAddressData(uint8 address, uint8 data)
{
	DisableGlobalIt();
	#ifdef HW_SPI
	 RF_NSEL_PIN = 0;
	 SSPBUF = address;
	 #asm
				nop	
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
	 #endasm
	 SSPBUF = data;
	 #asm
				nop	
				nop
				nop
				nop
				nop
				nop
				nop
				nop
				nop
	 #endasm
	#else
	#endif
	RF_NSEL_PIN = 1;
	EnableGlobalIt();
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 SpiReadWriteByte(uint8 data)
  +
  + DESCRIPTION:    sends and read 8 length data through the SPI port
  +
  +	INPUT:			data 
  +
  + RETURN:         received byte
  +
  + NOTES:          it controls the nSEL pin
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 SpiReadWriteByte(uint8 data)
{
	uint8 temp8;

	RF_NSEL_PIN = 0;
	#ifdef HW_SPI
 	 SSPBUF = data;						
	 #asm
		spirwb:	btfss 	c:_SSPSTAT,0
				bra 	spirwb	
	 #endasm
	 temp8 = SSPBUF;
	#else
	#endif
	RF_NSEL_PIN = 1;
	return temp8;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint16 SpiReadWriteWord(uint16 data)
  +
  + DESCRIPTION:    sends and read 16 length data through the SPI port
  +
  +	INPUT:			data 
  +
  + RETURN:         received word
  +
  + NOTES:          it controls the nSEL pin
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#pragma interrupt_level 1
uint16 SpiReadWriteWord(uint16 data)
{
	uint16 temp16;
	
	DisableGlobalIt();
	RF_NSEL_PIN = 0; 
	#ifdef HW_SPI
 	 SSPBUF = (uint8)(data >> 8);						
	 #asm
		spirww1: btfss 	c:_SSPSTAT,0
				 bra 	spirww1	
	 #endasm
	 temp16 = (uint16)(SSPBUF << 8);
 	 SSPBUF = (uint8)(data & 0x00FF);						
	 #asm
		spirww2: btfss 	c:_SSPSTAT,0
				 bra 	spirww2	
	 #endasm
	 temp16 += (uint16)(SSPBUF);
	#else
	#endif
	RF_NSEL_PIN = 1;
	EnableGlobalIt();
	return temp16;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint16 SpiReadWriteAddressData(uint8 address, uint8 data)
  +
  + DESCRIPTION:    sends and read 16 length data through the SPI port 
  +
  +	INPUT:			address - register address
  +					data 	- data
  +
  + RETURN:         received word
  +
  + NOTES:          it controls the nSEL pin
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint16 SpiReadWriteAddressData(uint8 address, uint8 data)
{
	uint16 temp16;

	DisableGlobalIt();
	RF_NSEL_PIN = 0; 
	#ifdef HW_SPI
 	 SSPBUF = address;						
	 #asm
		spirwad1: btfss c:_SSPSTAT,0
				  bra 	spirwad1	
	 #endasm
	 temp16 = (uint16)(SSPBUF << 8);
 	 SSPBUF = data;						
	 #asm
		spirwad2: btfss c:_SSPSTAT,0
				  bra 	spirwad2	
	 #endasm
	 temp16 += (uint16)(SSPBUF);
	#else
	#endif
	RF_NSEL_PIN = 1;
	EnableGlobalIt();
	return temp16;
}


#endif
