/*
** ============================================================================
**
** FILE
**  $Id: PIC.h,v 1.0 2007/06/20 krk Exp apache $
**
** DESCRIPTION
**  Contains the PIC specific declarations, IO declarations, type declarations
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#ifndef PIC_H
#define PIC_H

#include <htc.h>

/*------------------------------------------------------------------------*/
/*						APPLICATION DEFINITIONS							  */
/*------------------------------------------------------------------------*/
#define			CLOCK_RATE_8MHZ
#define			HW_SPI

                /* ======================================= *
                 *     T Y P E   D E C L A R A T I O N     *
                 * ======================================= */
//Only these type of variables are used in this software
#undef	uint8
#undef	sint8
#undef	uint16
#undef	sint16
#undef	uint32
#undef	sint32

#define	uint8	unsigned char
#define	sint8	signed char
#define	uint16	unsigned short
#define	sint16	signed short
#define	uint32	unsigned long
#define	sint32	signed long



typedef struct
{
unsigned int bit0 : 1;
unsigned int bit1 : 1;
unsigned int bit2 : 1;
unsigned int bit3 : 1;
unsigned int bit4 : 1;
unsigned int bit5 : 1;
unsigned int bit6 : 1;
unsigned int bit7 : 1;
} reg;

typedef union
{
   reg testreg;
   uint8 adat;
}reg_union;

typedef union
{
   reg_union bytes[2];
   uint16 adat;
}reg16_union;



                /* ======================================= *
                 *    PIC dependend 	M A C R O S        *
                 * ======================================= */
#define	EnableGlobalIt()			GIE = 1
#define	DisableGlobalIt()			GIE = 0
#define	EnablePeripheralIt()		PEIE = 1
#define	DisablePeripheralIt()		PEIE = 0
#define	EnableExt0It()				INT0IE = 1; INT0IF = 0
#define	DisableExt0It()				INT0IE = 0; INT0IF = 0
#define	SetExt0ItFallingEdge()		INTEDG0 = 0
#define SetExt0ItRisingEdge()		INTEDG0 = 1
#define EnableExt1It()				INT1IE = 1; INT1IF = 0
#define DisableExt1It()				INT1IE = 0; INT1IF = 0
#define SetExt1ItFallingEdge()		INTEDG1 = 0
#define SetExt1ItRisingEdge()		INTEDG1 = 1
#define EnableExt2It()				INT2IE = 1; INT2IF = 0
#define DisableExt2It()				INT2IE = 0; INT2IF = 0
#define SetExt2ItFallingEdge()		INTEDG2 = 0
#define SetExt2ItRisingEdge()		INTEDG2 = 1
#define EnablePbIt()				RBIE = 1
#define DisablePbIt()				RBIE = 0

#define	Set4MHzInternalRc()			OSCCON = 0x63
#define	Set8MHzInternalRc()			OSCCON = 0x73
#define	InternalRcInstable()		((OSCCON & 0x04) == 0)

#define DisableCCPModules()			CCP1CON = 0; CCP2CON = 0
#define	DisableComparator() 		CMCON = 0x07
#define DisableAdc()				ADCON0 = 0;	ADCON1 = 0x0F


                /* ======================================= *
                 *          D E F I N I T I O N S          *
                 * ======================================= */
#undef	TRUE
#undef 	FALSE
#undef	INPUT
#undef	OUTPUT

#define TRUE						(1)
#define FALSE						(0)
#define	INPUT						(1)
#define	OUTPUT						(0)

//I/O pin definitions
#define	LED1_PIN					(RA0)
#define	LED1_DIR					(TRISA0)
#define	LED2_PIN					(RA1)
#define	LED2_DIR					(TRISA1)
#define LED3_PIN					(RA2)
#define LED3_DIR					(TRISA2)	
#define LED4_PIN					(RA3)
#define LED4_DIR					(TRISA3)
#define BLED_PIN					(RA6)
#define BLED_DIR					(TRISA6)
#define	PB1_PIN						(RB4)
#define	PB1_DIR						(TRISB4)
#define	PB2_PIN						(RB5)
#define	PB2_DIR						(TRISB5)
#define PB3_PIN						(RB6)
#define PB3_DIR						(TRISB6)
#define PB4_PIN						(RB7)
#define PB4_DIR						(TRISB7)
#define TIN_PIN						(PB1_PIN)
#define TOUT_PIN					(LED1_PIN)
//RF chip
#define RF_NIRQ_PIN					(RB0)
#define RF_NIRQ_DIR					(TRISB0)
#define RF_NSEL_PIN					(RA5)
#define RF_NSEL_DIR					(TRISA5)
#define RF_GPIO0_PIN				(RC0)
#define RF_GPIO0_DIR				(TRISC0)
#define RF_GPIO1_PIN				(RC1)
#define RF_GPIO1_DIR				(TRISC1)
#define RF_GPIO2_PIN				(RA7)
#define RF_GPIO2_DIR				(TRISC7)
//Testcard EEPROM
#define EE_NSEL_PIN					(RB2)
#define EE_NSEL_DIR					(TRISB2)

//SPI port
#define MOSI_PIN					(RC5)
#define MOSI_DIR					(TRISC5)
#define MISO_PIN					(RC4)
#define MISO_DIR					(TRISC4)
#define SCK_PIN						(RC3)
#define SCK_DIR						(TRISC3)

//definitions for the SSPCON1 register
#define	ENABLE_MSSP					0x20
#define	SCK_HIGH_DURING_IDLE		0x10
#define	SCK_LOW_DURING_IDLE			0x00
#define	SPI_MASTER_FOSC4			0x00


                /* ======================================= *
                 *            V A R I A B L E S            *
                 * ======================================= */


                /* ======================================= *
                 *  F U N C T I O N   P R O T O T Y P E S  *
                 * ======================================= */
void SetHwMasterSpi(void);
void SpiWrite(uint8 data);
uint8 SpiReadWrite(uint8 data);
void SpiWriteByte(uint8 data);
void SpiWriteWord(uint16 data);
void SpiWriteAddressData(uint8 address, uint8 data);
uint8 SpiReadWriteByte(uint8 data);
uint16 SpiReadWriteWord(uint16 data);
uint16 SpiReadWriteAddressData(uint8 address, uint8 data);

#endif
