/*
** ============================================================================
**
** FILE
**  $Id: dog_lcd.c,v 1.0 2007/08/28 krk Exp apache $
**
** DESCRIPTION
**  Main file for controlling the DOG LCD thorugh SPI interface.
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#include "dog_glcd.h"

#define LCD_BACKLIGHT_IS_USED

extern const uint8 ascii_table5x7[][5];
extern const uint8 iai_logo128x16[];

extern const uint8 pict1[];
extern const uint8 pict2[];
uint8 CurrentLine, CurrentChPos, CurrentPage, CurrentColumn;


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdDelay_2ms(void)
  +
  + DESCRIPTION:    wait about 20ms, it uses only a for cycle
  +
  + RETURN:         None
  +
  + NOTES:          the delay depends on the clock of the MCU
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdDelay_2ms(void)
{
	uint16 delay;

	for(delay=0;delay<380;delay++);
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdInit(void)
  +
  + DESCRIPTION:    Initialize the LCD for 3.3V operation voltage and SPI comm.
  +
  + RETURN:         None
  +
  + NOTES:          it can be called only 40ms later than the VDD stabilized
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdInit(void)
{
	LCD_RESET_DIR	= OUTPUT;
	LCD_NSEL_DIR	= OUTPUT;
	LCD_A0_DIR		= OUTPUT;
	#ifdef LCD_BACKLIGHT_IS_USED
	 LCD_BL_DIR 	= OUTPUT;
	 LCD_BL_PIN 	= 1;
	#endif
	LCD_NSEL_PIN 	= 1;
	LCD_A0_PIN 		= 0;
	//perform reset
	LCD_RESET_PIN 	= 0;
	LcdDelay_2ms();
	LCD_RESET_PIN 	= 1;
	LcdDelay_2ms();
	LcdDelay_2ms();
	LcdDelay_2ms();


	LCD_NSEL_PIN = 0;
	SpiWrite(0x40);			//display start line 0
	SpiWrite(0xA1);			//ADC reverse
	SpiWrite(0xC0);			//normal COM0~COM63
	SpiWrite(0xA6);			//display normal
	SpiWrite(0xA2);			//set bias 1/9 (Duty 1/65)
	SpiWrite(0x2F);			//booster, regulator and follower on
	SpiWrite(0xF8);			//set internal bosster to 4x
	SpiWrite(0x00);			
	SpiWrite(0x27);			//contrast set
	SpiWrite(0x81);			
	SpiWrite(0x16);			
	SpiWrite(0xAC);			//no indicator
	SpiWrite(0x00);			
	LCD_NSEL_PIN = 1;
	LcdOff();
	LcdClearDisplay();
	LcdOn();
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdOn(void)
  +
  + DESCRIPTION:    turns on the LCD screen
  +
  + RETURN:         None
  +
  + NOTES:         
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdOn(void)
{
	LCD_NSEL_PIN = 0;
	SpiWrite( 0xAF );			
	LCD_NSEL_PIN = 1;	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdOff(void)
  +
  + DESCRIPTION:    turns off the LCD screen (the DDRAM content will be kept)
  +
  + RETURN:         None
  +
  + NOTES:         
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdOff(void)
{
	LCD_NSEL_PIN = 0;
	SpiWrite( 0xAE );			
	LCD_NSEL_PIN = 1;	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdSetPage(uint8 data)
  +
  + DESCRIPTION:    it sets the page address (0...7)
  +
  + INPUT:			page address (the LCD is separated to 8x8 rows
  +					and 8 row is calles as a page)
  +					page0 is the top 8 rows
  +
  + RETURN:         None
  +
  + NOTES:         
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdSetPage(uint8 data)
{
	LCD_NSEL_PIN = 0;
	SpiWrite( 0xB0 | data );			
	LCD_NSEL_PIN = 1;	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdSetColumn(uint8 data)
  +
  + DESCRIPTION:    it sets the column address (0...127)
  +
  + INPUT:			address of the column
  +
  + RETURN:         None
  +
  + NOTES:         
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdSetColumn(uint8 data)
{
	LCD_NSEL_PIN = 0;
	SpiWrite( 0x10 | ((data & 0x70) >> 4) );			
	SpiWrite( 0x00 | (data & 0x0F) );			
	LCD_NSEL_PIN = 1;	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdClearDisplay(void)
  +
  + DESCRIPTION:    it clears the display (the content of the DDRAM!)
  +
  + INPUT:			None
  +
  + RETURN:         None
  +
  + NOTES:         
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdClearDisplay(void)
{
	uint8 page;

	for(page=1;page<9;page++)
	{
		LcdClearLine(page);
	}	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdClearLine(uint8 page)
  +
  + DESCRIPTION:    it clears one line on the LCD
  +
  + INPUT:			None
  +
  + RETURN:         None
  +
  + NOTES:         
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdClearLine(uint8 line)
{
	uint8 column;

	if( (line < LCD_MIN_LINE) || (line > LCD_MAX_LINE) )
	{
		return;
	}
	//select the page
	LcdSetPage(line-1);
	//set to the first column
	LcdSetColumn(0);
	//set A0 to 1 -> access to the DDRAM
	LCD_A0_PIN 	 = 1;
	LCD_NSEL_PIN = 0;
	for(column=0;column<128;column++)
	{
		//clear the selected column
		SpiWrite(0x00);
	}
	LCD_A0_PIN 	 = 0;
	LCD_NSEL_PIN = 1;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdSetCharCursor(uint8 line, uint8 ch_pos)
  +
  + DESCRIPTION:    it sets the character position
  +
  + INPUT:			line   - number of the line (1...8
  +							 the LCD is divided to 8 lines
  +						     line1 is the top line
  +					ch_pos - character position
  +							 up to 21 character (1...21) could be in a line	
  +							 character 1 is the first on left hand side
  +
  + RETURN:         TRUE   - operation was successfull
  +					FALSE  - operation was ignored
  +
  + NOTES:          If the position is invalid, the function returns without
  +					changing the registers.
  +					The function sets the CurrentLine, CurrentChPos variables!
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 LcdSetCharCursor(uint8 line, uint8 ch_pos)
{
	//check whether the line and ch_pos valid or not
	if( ((line < LCD_MIN_LINE) || (line > LCD_MAX_LINE)) || ((ch_pos < LCD_MIN_CHAR) || (ch_pos > LCD_MAX_CHAR)) ) 
	{
		return FALSE;
	}

	//set page address
	LcdSetPage(line-1);
	//set column address
	LcdSetColumn( ((ch_pos-1)*6)+1 );
	CurrentLine = line;
	CurrentChPos = ch_pos;

	return TRUE;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdPutInvCh(uint8 ch)
  +
  + DESCRIPTION:    it writes one character INVERTED to the next position
  +
  + INPUT:			ch	   - the character needs to be writen
  +
  + RETURN:         None
  +
  + NOTES:          if the position is invalid (the line is full) it writes 
  +					the character from the begining of the line.
  +					The function increments the CurrentChPos variable!
  +					If the character invalid, it prints a space char.
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdPutInvCh(uint8 ch)
{
	uint8 column, data, temp8;
	
	if( (ch > ASCII_5X7_MAX) || (ch < ASCII_5X7_MIN) )
	{
		data = ' ';
	}
	else
	{
		data = ch;
	}

	//write character
	LCD_A0_PIN 	 = 1;			//set A0 to 1 -> access to the DDRAM
	LCD_NSEL_PIN = 0;
	for(column=0;column<5;column++)
	{
		//write column data
		temp8 = ascii_table5x7[data - ASCII_5X7_MIN][column];
		temp8 ^= 0xFF;
		temp8 &= 0x7F;
		SpiWrite( temp8 );
	}
	//space between the characters
	SpiWrite(0x7F);
	LCD_A0_PIN 	 = 0;
	LCD_NSEL_PIN = 1;
	if(	++CurrentChPos > LCD_MAX_CHAR )
	{//end of the line -> set cursor to the beginning of the line
		CurrentChPos = 1;
		LcdSetColumn( 1 );
	}	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdPutChar(uint8 line, uint8 ch_pos, uint8 ch)
  +
  + DESCRIPTION:    it writes one character to the next position
  +
  + INPUT:			ch	   - the character needs to be writen
  +
  + RETURN:         None
  +
  + NOTES:          if the position is invalid (the line is full) it writes 
  +					the character from the begining of the line.
  +					The function increments the CurrentChPos variable!
  +					If the character invalid, it prints a space char.
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdPutCh(uint8 ch)
{
	uint8 column, data;
	
	if( (ch > ASCII_5X7_MAX) || (ch < ASCII_5X7_MIN) )
	{
		data = ' ';
	}
	else
	{
		data = ch;
	}

	//write character
	LCD_A0_PIN 	 = 1;			//set A0 to 1 -> access to the DDRAM
	LCD_NSEL_PIN = 0;
	for(column=0;column<5;column++)
	{
		//write column data
		SpiWrite(ascii_table5x7[data - ASCII_5X7_MIN][column]);
	}
	//space between the characters
	SpiWrite(0);
	LCD_A0_PIN 	 = 0;
	LCD_NSEL_PIN = 1;
	if(	++CurrentChPos > LCD_MAX_CHAR )
	{//end of the line -> set cursor to the beginning of the line
		CurrentChPos = 1;
		LcdSetColumn( 1 );
	}	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdPutChar(uint8 line, uint8 ch_pos, uint8 ch)
  +
  + DESCRIPTION:    it write one character to the requested position
  +
  + INPUT:			line   - number of the line (1...8
  +							 the LCD is divided to 8 lines
  +						     line1 is the top line
  +					ch_pos - character position
  +							 up to 21 character (1...21) could be in a line	
  +							 character 1 is the first on left hand side
  +                 ch	   - the character needs to be writen
  +
  + RETURN:         None
  +
  + NOTES:          If the position is invalid, the function returns without
  +					changing the registers.
  +					The function sets the CurrentLine, CurrentChPos variables!
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdPutChar(uint8 line, uint8 ch_pos, uint8 ch)
{
	if( LcdSetCharCursor(line, ch_pos) == FALSE )
	{
		return;
	}
	LcdPutCh(ch);
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdWriteLine(uint8 line, uint8 * text)
  +
  + DESCRIPTION:    it writes one complete line
  +
  + INPUT:			line   - number of the line (1...8
  +							 the LCD is divided to 8 lines
  +						     line1 is the top line
  +					text   - address of the string needs to be written
  +
  + RETURN:         None
  +
  + NOTES:          If the line is invalid, the function returns without any changes.
  +					The function doesn't set the CurrentLine, CurrentChPos variables!
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdWriteLine(uint8 line, uint8 * text)
{
	uint8 i,data,column;

	if( (line < LCD_MIN_LINE) || (line > LCD_MAX_LINE) )
	{
		return;
	}
	//set page address
	LcdSetPage( line-1 );
	//set column address
	LcdSetColumn( 0 );

	for(i=0;i<21;i++)	
	{
		if( text[i] > ASCII_5X7_MAX || text[i] < ASCII_5X7_MIN)
			data = ' ';
		else
			data = text[i];

		//write character
		LCD_A0_PIN 	 = 1;			//set A0 to 1 -> access to the DDRAM
		LCD_NSEL_PIN = 0;
		for(column=0;column<5;column++)
		{
			//write column data
			SpiWrite(ascii_table5x7[data - ASCII_5X7_MIN][column]);
		}
		//space between the characters
		SpiWrite(0);
		LCD_A0_PIN 	 = 0;
		LCD_NSEL_PIN = 1;
	}	
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 LcdSetPictureCursor(uint8 page, uint8 column)
  +
  + DESCRIPTION:    it sets the character position
  +
  + INPUT:			page   - number of the pages (1...8
  +							 the LCD is divided to 8 pages
  +						     page1 is the top page
  +					column - number of start column
  +							 column1 is the left one
  +
  + RETURN:         TRUE   - operation was successfull
  +					FALSE  - operation was ignored
  +
  + NOTES:          If the position is invalid, the function returns without
  +					changing the registers.
  +					The function sets the CurrentLine, CurrentChPos variables!
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 LcdSetPictureCursor(uint8 page, uint8 column)
{
	//check whether the input parameters are correct or not
	if( ((page < LCD_MIN_LINE) || (page > LCD_MAX_LINE)) || ((column < LCD_MIN_COLUMN) || (column > LCD_MAX_COLUMN)) ) 
	{
		return FALSE;
	}

	//set page address
	LcdSetPage(page-1);
	//set column address
	LcdSetColumn( column-1 );
	CurrentPage = page;
	CurrentColumn = column;

	return TRUE;
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void LcdDrawPicture(const uint8 * picture)
  +
  + DESCRIPTION:    it draw a picture
  +
  + INPUT:			picture - address of the picture (must be stored in the 
  +							  FLASH
  +
  + RETURN:         None
  +
  + NOTES:          The uint8 LcdSetPictureCursor(uint8 page, uint8 column) function
  +					has to be called before calling this function!
  +					The function changes the CurrentPage and CurrentColumn variables!
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void LcdDrawPicture(const uint8 * picture)
{
uint8 p,pages,page,i,col;
uint16 j;

	p = picture[LCD_PIC_PAGE_NMBR];
	//check wheter there are enough column for the picture or not
	if( (LCD_MAX_COLUMN - CurrentColumn + 1) < picture[LCD_PIC_COLUMN_NMBR] )
	{//there are not enough space for the pic -> limit to the available space
		col = (LCD_MAX_COLUMN - CurrentColumn + 1);
	}
	else
	{
		col = picture[LCD_PIC_COLUMN_NMBR];
	}
	//check wheter there are enough pages for the picture or not
	if( (LCD_MAX_LINE - CurrentPage + 1) < picture[LCD_PIC_PAGE_NMBR] )
	{
		page = (LCD_MAX_LINE - CurrentPage + 1);
	}
	else
	{
		page = picture[LCD_PIC_PAGE_NMBR];
	}

	//draw the picture
	for(pages=0;pages<page;pages++)
	{
		LCD_A0_PIN 	 = 1;			//set A0 to 1 -> access to the DDRAM
		LCD_NSEL_PIN = 0;
		j = LCD_PIC_ADDRESS_OFFSET + pages;
		for(i=0;i<col;i++)
		{
			//write column data
			SpiWrite( picture[j] );
			j += p;
		}
		LCD_A0_PIN 	 = 0;
		LCD_NSEL_PIN = 1;
		//set next page
		LcdSetPictureCursor( ++CurrentPage,CurrentColumn );
	}
}
