/*
** ============================================================================
**
** FILE
**  $Id: main.c,v 1.0 2007/11/28 krk Exp apache $
**
** DESCRIPTION
**  Main file of the IA443x demo
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

/*------------------------------------------------------------------------*/
/*						INCLUDE											  */
/*------------------------------------------------------------------------*/
#include "PIC.c"
#include "PIC.h" 
#include "IA443x_rf.h"
#include "uart.h"
#include "pic18_timers.h"
#include "dog_glcd.h"
#include "IA443x_demo.h"
#include "global_definitions.h"
#ifdef TEST_FUNCTIONS_ENABLED
	#include "IA443x_test.h"
#endif
#include <stdio.h>
#include <string.h>


/*------------------------------------------------------------------------*/
/*						GLOBAL variables								  */
/*------------------------------------------------------------------------*/
#ifdef TEST_FUNCTIONS_ENABLED
	volatile S_TEST sDemo;
#endif
extern volatile uint8 fUartCommandReceived;		
extern uint16 SpiCommand;

/*------------------------------------------------------------------------*/
/*						FUNCTION prototypes								  */
/*------------------------------------------------------------------------*/
void _InitHw(void);

/*------------------------------------------------------------------------*/
/*						   MAIN routine									  */
/*------------------------------------------------------------------------*/
void main(void)
{
	//initialize the HW
	_InitHw();
	//initialize the demo
	DemoInit();
	//main loop
	while(1)
	{
#ifdef TEST_FUNCTIONS_ENABLED
		//run the demo state machine 
		switch( sDemo )
		{
			case DEMO:
				DemoStateMachine();
			break;

			case PER:
				TestPerStateMachine();
			break;

			case BER:
				TestBerStateMachine();
			break;

			case CW:
				TestCwStateMachine( MOD_TYPE_CW );
			break;	

			case PN9:
				TestCwStateMachine( MOD_PN9 );
			break;

			default:
			break;		
		}
#else
		DemoStateMachine();
#endif

		//if uart command received, handle it
		if(fUartCommandReceived == TRUE)
		{
			fUartCommandReceived = FALSE;
			UartCommandInterpreter();
		}
	}
}

/*------------------------------------------------------------------------*/
/*				   FUNCTIONs used in the main.c						  	  */
/*------------------------------------------------------------------------*/
void _InitHw(void)
{
	//set 8MHz internal oscillator
	Set8MHzInternalRc();
	while( InternalRcInstable() );
	//disable all nonused peripherals
	//Capcture compare module: OFF
	DisableCCPModules();
	//Comparator: OFF
	DisableComparator(); 
	//ADC: OFF, all pins are I/O
	DisableAdc();

	//initialize I/O port directions
	LED1_DIR 		= OUTPUT;
	LED2_DIR 		= OUTPUT;
	LED3_DIR 		= OUTPUT;
	LED4_DIR 		= OUTPUT;
	BLED_DIR		= OUTPUT;
	PB1_DIR 		= INPUT;
	PB2_DIR 		= INPUT;
	PB3_DIR			= INPUT;
	PB4_DIR			= INPUT;
	RF_NIRQ_DIR 	= INPUT;
	RF_NSEL_DIR 	= OUTPUT;
	EE_NSEL_DIR		= OUTPUT;
	LCD_RESET_DIR	= OUTPUT;
	LCD_NSEL_DIR	= OUTPUT;
	LCD_A0_DIR		= OUTPUT;

	//default I/O port
	LED1_PIN 		= 0;
	LED2_PIN 		= 0;
	LED3_PIN 		= 0;
	LED4_PIN 		= 0;
	BLED_PIN 		= 0;
	RF_NSEL_PIN 	= 1;
	EE_NSEL_PIN		= 1;
	LCD_NSEL_PIN 	= 1;
	LCD_A0_PIN 		= 0;
	LCD_RESET_PIN	= 1;

	//initialize SPI port
    SetHwMasterSpi();

	//initialize the UART peripherals
	UartInit();

	//initialize the LCD
	LcdInit();

	//enable interrupts
	EnablePeripheralIt();
	DisablePbIt();
	EnableGlobalIt();
}
