/*
** ============================================================================
**
** FILE
**  $Id: menu.c,v 1.0 2007/09/06 krk Exp apache $
**
** DESCRIPTION
**  Realize the menu operation
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#include <stdio.h>
#include <string.h>
#include "menu.h"
#include "global_definitions.h"
#include "IA443x_demo.h"
#include "dog_glcd.h"
#include "pic18_timers.h"
#include "IAI_EBID.h"

/*------------------------------------------------------------------------*/
/*						GLOBAL variables, constants						  */
/*------------------------------------------------------------------------*/
uint8 ParamToChange, ButtonPressed;

//extern variables, constans
extern const uint8 FirmwareVersion[];
extern const volatile DEMO_FREQ demo_freqs[MAX_FREQ_SETTING];

extern const uint8 full_battery8x8[];
extern const uint8 low_battery8x8[];
extern const uint8 ant1_inverted13x8[];
extern const uint8 ant2_inverted15x8[];
extern const uint8 ok_inverted16x8[];
extern const uint8 rx_inverted16x8[];
extern const uint8 tx_inverted16x8[];
extern const uint8 txon_inverted29x8[];
extern const uint8 iai_logo128x16[];
extern const uint8 txon_inverted32x8[];
extern const uint8 txoff_inverted32x8[];
extern const uint8 setting_inverted48x8[];
extern const uint8 minus_inverted16x8[];
extern const uint8 plus_inverted16x8[];
extern const uint8 up_down_inverted16x8[];
extern const uint8 go_inverted16x8[];

extern EBID_DATA	TestcardData;
extern volatile DEMO_PARAMETERS demo_param;
extern volatile uint8 customer_name[CUSTOMER_NAME_SIZE];
#ifdef TEST_FUNCTIONS_ENABLED
 extern volatile uint8 rf_setting;
#endif

/*------------------------------------------------------------------------*/
/*						LOCAL function prototypes						  */
/*------------------------------------------------------------------------*/
void PrintRssiToLcd(void);

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  
  +
  + DESCRIPTION:    
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
uint8 PerformMenu(MENU_ITEM menu_item)
{
uint8 i,string[6];
uint8 lcd_data[22];

	switch( menu_item )
	{
		case sMenuStartScreen:
			//clear LCD
			LcdClearDisplay();
			//draw startup screen
			LcdSetPictureCursor(LCD_LINE_1,1);
			LcdDrawPicture(iai_logo128x16);
			memcpy(lcd_data,"       IA443x        ",21);
			LcdWriteLine(LCD_LINE_4,lcd_data);	
			memcpy(lcd_data,"  Packet Error Rate  ",21);			
			LcdWriteLine(LCD_LINE_5,lcd_data);	
			memcpy(lcd_data,"    Demonstration    ",21);			
			LcdWriteLine(LCD_LINE_6,lcd_data);	
			sprintf(lcd_data,"  (Firmware: %s)   ",FirmwareVersion);
			LcdWriteLine(LCD_LINE_8,lcd_data);	
			printf("\r\n\r\nINTEGRATION, IA443x Packet Error Rate Demonstration\r\n");
			printf("Firmware: %s\r\n",FirmwareVersion);
		break;

		case sMenuXTALError:
			LcdClearDisplay();
			printf("\r\nERROR: XTAL startup error!");
			printf("\r\nTry a different Testcard!\r\n");
			memcpy(lcd_data,"ERROR: XTAL startup! ",21);
			LcdWriteLine(LCD_LINE_2,lcd_data);
			memcpy(lcd_data,"   Try a different   ",21);
			LcdWriteLine(LCD_LINE_5,lcd_data);
			memcpy(lcd_data,"      Testcard!      ",21);
			LcdWriteLine(LCD_LINE_6,lcd_data);
			while(1);
		break;

		case sMenuPORTimeout:
			//clear LCD
			LcdClearDisplay();
			printf("\r\nERROR: POR timeout! Insert a Testcard or try a different one!\r\n");
			memcpy(lcd_data,"ERROR: POR timeout!  ",21);
			LcdWriteLine(LCD_LINE_2,lcd_data);
			memcpy(lcd_data,"Insert a Testcard or ",21);
			LcdWriteLine(LCD_LINE_5,lcd_data);
			memcpy(lcd_data,"try a different one! ",21);
			LcdWriteLine(LCD_LINE_6,lcd_data);
		break;

		case sMenuTestcardOrEEPROMError:
			//clear LCD
			LcdClearDisplay();
			//EEPROM is empty
			printf("\r\nERROR: no Testcard inserted, or the board identification");  
			printf("\r\nEEPROM is missing or empty!");
			printf("\r\nTry a different Testcard!\r\n");
			memcpy(lcd_data,"ERROR: no Testcard or",21);
			LcdWriteLine(LCD_LINE_1,lcd_data);
			memcpy(lcd_data,"   the board ident.  ",21);
			LcdWriteLine(LCD_LINE_2,lcd_data);
			memcpy(lcd_data,"EEPROM missing/empty!",21);
			LcdWriteLine(LCD_LINE_3,lcd_data);
			memcpy(lcd_data,"   Try a different   ",21);
			LcdWriteLine(LCD_LINE_5,lcd_data);
			memcpy(lcd_data,"      Testcard!      ",21);
			LcdWriteLine(LCD_LINE_6,lcd_data);
		break;

		case sMenuNotSupportedChip:
			//clear LCD
			LcdClearDisplay();
			//EEPROM is empty
			printf("\r\nERROR: the chip is not supported");  
			printf("\r\nby the current firmware!");
			printf("\r\nTry a different Testcard!\r\n");
			memcpy(lcd_data,"ERROR: the chip isn't",21);
			LcdWriteLine(LCD_LINE_2,lcd_data);
			memcpy(lcd_data,"supported by the FW! ",21);
			LcdWriteLine(LCD_LINE_3,lcd_data);
			memcpy(lcd_data,"   Try a different   ",21);
			LcdWriteLine(LCD_LINE_5,lcd_data);
			memcpy(lcd_data,"      Testcard!      ",21);
			LcdWriteLine(LCD_LINE_6,lcd_data);
		break;

		case sMenuInitSettingsPage:
			//get default settings from the PIC EEPROM
			//addresses
			demo_param.self_address = (uint8)(TestcardData.serial_nmbr & 0x00FF);
			if( EEPROM_READ(EE_ADDRESS_FIRST_PLUG) == 0 )
			{
				demo_param.destination_address = (uint8)(TestcardData.serial_nmbr & 0x00FF);
			}
			else
			{
				demo_param.destination_address = EEPROM_READ(EE_ADDRESS_DID);
			}
			//frequency
			demo_param.freq_number = EEPROM_READ(EE_ADDRESS_DEFAULT_FREQ);
			//data rate
			demo_param.datarate = EEPROM_READ(EE_ADDRESS_DEFAULT_BR);

			//get customer name
			for(i=0;i<CUSTOMER_NAME_SIZE;i++)
			{
				customer_name[i] = EEPROM_READ(i);
			}
			//set variables
			ParamToChange = PARAM_DR; 
			ButtonPressed = BUTTON_NON;

		break;

		case sMenuSettingsScreen:
			//clear LCD
			LcdClearDisplay();
			//DRAW welcome-setting screen
			memcpy(lcd_data,"                     ",21);
			sprintf(lcd_data,"%s",customer_name);
			LcdWriteLine(LCD_LINE_1,lcd_data);
			//setting page
			memcpy(lcd_data,"SETUP PARAMETERS:    ",21);
			LcdWriteLine(LCD_LINE_2,lcd_data);	
			//check whether the parameter select pushbutton is pressed or not
			if( ButtonPressed == BUTTON_1 )
			{
				if( ParamToChange < PARAM_DID )
				{
					ParamToChange++;
				}
				else
				{
					ParamToChange = PARAM_DR;
				}
			}
			//draw push button labels
			LcdSetPictureCursor(8,13);			
			LcdDrawPicture(up_down_inverted16x8);
			LcdSetPictureCursor(8,43);			
			LcdDrawPicture(plus_inverted16x8);
			LcdSetPictureCursor(8,73);			
			LcdDrawPicture(minus_inverted16x8);
			LcdSetPictureCursor(8,103);			
			LcdDrawPicture(go_inverted16x8);

			//check whether the parameter has to be changed or not
			switch( ParamToChange)
			{
				case PARAM_DR:
					//check PBs
					if( ButtonPressed == BUTTON_2 )
					{//inc
						//change data rate setting
						if( demo_param.datarate < (NMBR_OF_SAMPLE_SETTING - 1) )
						{
							demo_param.datarate++;
						}
						else
						{
							demo_param.datarate = 0;
						}
					}
					if( ButtonPressed == BUTTON_3 )
					{//dec 
						//change data rate setting
						if( demo_param.datarate == 0 )
						{
							demo_param.datarate = NMBR_OF_SAMPLE_SETTING - 1;
						}
						else
						{
							demo_param.datarate--;
						}
					}
				break;

				case PARAM_FREQ:
					//check PBs
					if( ButtonPressed == BUTTON_2 )
					{//inc
						if( demo_param.freq_number < (MAX_FREQ_SETTING - 1) )
						{
							demo_param.freq_number++;
						}
						else
						{
							demo_param.freq_number = 0;
						}
					}
					if( ButtonPressed == BUTTON_3 )
					{//dec 
						if( demo_param.freq_number == 0 )
						{
							demo_param.freq_number = MAX_FREQ_SETTING - 1;
						}
						else
						{
							demo_param.freq_number--;
						}
					}
				break;
				
				case PARAM_DID:
					//check PBs
					if( ButtonPressed == BUTTON_2 )
					{//inc
						if( demo_param.destination_address < 255 )
						{
							demo_param.destination_address++;
						}
						else
						{
							demo_param.destination_address = 0;
						}
					}
					if( ButtonPressed == BUTTON_3 )
					{//dec 
						if( demo_param.destination_address == 0 )
						{
							demo_param.destination_address = 255;
						}
						else
						{
							demo_param.destination_address--;
						}
					}
				break;
			}

			GetDataRate( demo_param.datarate, &string[0]);		
			string[3] = 0;
			sprintf(lcd_data,"  Datarate: %skbps ",string);
			LcdWriteLine(LCD_LINE_3,lcd_data);	
			sprintf(lcd_data,"  Frequency: %3uMHz  ",demo_freqs[demo_param.freq_number].freq_decimal);
			LcdWriteLine(LCD_LINE_4,lcd_data);	
			sprintf(lcd_data,"  Dest. ID:  %5u   ",demo_param.destination_address);
			LcdWriteLine(LCD_LINE_5,lcd_data);	
			sprintf(lcd_data,"  (Self ID:  %5u)  ",demo_param.self_address);
			LcdWriteLine(LCD_LINE_6,lcd_data);	
			memcpy(lcd_data,"---------------------",21);
			LcdWriteLine(LCD_LINE_7,lcd_data);	

			//draw the arrow
			switch( ParamToChange)
			{
				case PARAM_DR: 		LcdSetCharCursor(3,1);	break;
				case PARAM_FREQ: 	LcdSetCharCursor(4,1);	break;
				case PARAM_DID:		LcdSetCharCursor(5,1);	break;
			}
			LcdPutCh('}'+1);
		break;

		case sMenucheckButtonsForSettings:
			if( PB1_PIN == 0 )
			{
				//wait for release the button
				while( PB1_PIN == 0);
				ButtonPressed = BUTTON_1;
				return BUTTON_1;
			}

			if( PB2_PIN == 0 )
			{
				//wait for release the button
				while( PB2_PIN == 0);
				ButtonPressed = BUTTON_2;
				return BUTTON_2;
			}

			if( PB3_PIN == 0 )
			{
				//wait for release the button
				while( PB3_PIN == 0);
				ButtonPressed = BUTTON_3;
				return BUTTON_3;
			}
			//run demo
			if( PB4_PIN == 0 )
			{
				//wait for release the button
				while( PB4_PIN == 0);
				ButtonPressed = BUTTON_4;
				return BUTTON_4;
			}
		break;

		case sMenuRestartDemo:
			//clear LCD
			LcdClearDisplay();
			//put info to the LCD
			LcdSetPictureCursor(1,12);
			LcdDrawPicture(tx_inverted16x8);
			LcdSetPictureCursor(1,41);
			LcdDrawPicture(rx_inverted16x8);

			LcdSetPictureCursor(8,3);
			LcdDrawPicture(txon_inverted32x8);
			LcdSetPictureCursor(8,73);
			LcdDrawPicture(setting_inverted48x8);

			memcpy(lcd_data,"---------------------",21);
			LcdWriteLine(LCD_LINE_2,lcd_data);	
			//LcdWriteLine(LCD_LINE_7,lcd_data);	


			//print chip, freq, data rate
			switch( TestcardData.chip_type )
			{
				case IA4431:
					sprintf(lcd_data,"IA4431               ");
				break;

				case IA4432:
				default:
					sprintf(lcd_data,"IA4432               ");
				break;
			}
			sprintf(&lcd_data[8],"SID%03u DID%03u",demo_param.self_address,demo_param.destination_address);
			printf("%s\r\n",lcd_data);
			LcdWriteLine(LCD_LINE_4,lcd_data);
			GetDataRate( demo_param.datarate, &string[0]);		
			string[3] = 0;
			LcdSetPictureCursor(1,72);
			LcdDrawPicture(ant1_inverted13x8);
			switch( TestcardData.ant_type )
			{
				case ANT_DIVERSITY:
					sprintf(lcd_data,"%sk/%uMHz   ANT_DIV",string,demo_freqs[demo_param.freq_number].freq_decimal);
					LcdSetPictureCursor(1,98);
					LcdDrawPicture(ant2_inverted15x8);
				break;

				case SMA_TRX_SWITCH:
					sprintf(lcd_data,"%sk/%uMHz  TX/RX-SW",string,demo_freqs[demo_param.freq_number].freq_decimal);
				break;

				case SEPARATE_SMA:
				default:
					sprintf(lcd_data,"%sk/%uMHz NO-ANTDIV",string,demo_freqs[demo_param.freq_number].freq_decimal);
				break;
			}
			LcdWriteLine(LCD_LINE_5,lcd_data);
			//save basic settings
			EEPROM_WRITE(EE_ADDRESS_FIRST_PLUG, 1);
			EEPROM_WRITE(EE_ADDRESS_DEFAULT_BR, demo_param.datarate);
			EEPROM_WRITE(EE_ADDRESS_DEFAULT_FREQ, demo_param.freq_number);
			EEPROM_WRITE(EE_ADDRESS_DID, demo_param.destination_address);
		break;

		case sMenuStartAutoTx:
			memcpy(lcd_data,"                     ",21);
			LcdWriteLine(LCD_LINE_8,lcd_data);	
			LcdSetPictureCursor(8,3);
			LcdDrawPicture(txoff_inverted32x8);
		break;

		case sMenuPingTxError:
			//report the result on the LCD
			sprintf(lcd_data,"PING TRANSMIT ERROR! ");
			LcdWriteLine(LCD_LINE_7,lcd_data);
			//report the result to the UART
			printf("PING TRANSMIT ERROR!\r\n>");
		break;

		case sMenuPingReceived:
			//print the RSSI result to the screen	
			PrintRssiToLcd();
		break;

		case sMenuAckSent:
			//report the result on the LCD
			memcpy(lcd_data,"ACK SENT             ",21);
			LcdWriteLine(LCD_LINE_7,lcd_data);
			//report the result to the UART
			printf("ACK SENT!\r\n>");
		break;

		case sMenuAckTxError:
			//report the result on the LCD
			memcpy(lcd_data,"ACK TX ERROR!        ",21);
			LcdWriteLine(LCD_LINE_7,lcd_data);
			//report the result to the UART
			printf("ACK TX ERROR!\r\n>");
		break;

		case sMenuDeleteMessageRow:
			memcpy(lcd_data,"                     ",21);
			LcdWriteLine(LCD_LINE_7,lcd_data);
		break;

		case sMenuAckReceivedMessage:
			sprintf(lcd_data,"ACK RECEIVED!        ");
			LcdWriteLine(LCD_LINE_7,lcd_data);
		break;

		case sMenuAckReceivingErrorMessage:
			sprintf(lcd_data,"ACK RECEIVING ERROR! ");
			LcdWriteLine(LCD_LINE_7,lcd_data);
		break;

#ifdef TEST_FUNCTIONS_ENABLED
		case sMenuPERStartScreen:
			//TODO: welcome screens
			printf("PER test\r\n>");
			LcdClearDisplay();
			memcpy(lcd_data,"      PER TEST       ",21);
			LcdWriteLine(LCD_LINE_4,lcd_data);	
			GetDataRate( rf_setting, &string[0]);		
			string[3] = 0;
			sprintf(lcd_data,"Datarate: %skbps   ",string);
			LcdWriteLine(LCD_LINE_1,lcd_data);
			sprintf(lcd_data,"Frequency: %3uMHz    ",GetFreq(demo_param.freq_number));
			LcdWriteLine(LCD_LINE_2,lcd_data);		
		break;
		
		case sMenuBERStartScreen:
			printf("BER test\r\n>");
			LcdClearDisplay();
			memcpy(lcd_data,"      BER TEST       ",21);
			LcdWriteLine(LCD_LINE_4,lcd_data);
			GetDataRate( rf_setting, &string[0]);		
			string[3] = 0;
			sprintf(lcd_data,"Datarate: %skbps   ",string);
			LcdWriteLine(LCD_LINE_1,lcd_data);	
			sprintf(lcd_data,"Frequency: %3uMHz    ",GetFreq(demo_param.freq_number));
			LcdWriteLine(LCD_LINE_2,lcd_data);	
		break;

		case sMenuCWStartScreen:
			LcdClearDisplay();
			memcpy(lcd_data,"       CW TEST       ",21);
			printf("CW test\r\n>"); 
			LcdWriteLine(LCD_LINE_4,lcd_data);
			GetDataRate( rf_setting, &string[0]);		
			string[3] = 0;
			sprintf(lcd_data,"Datarate: %skbps   ",string);
			LcdWriteLine(LCD_LINE_1,lcd_data);	
			sprintf(lcd_data,"Frequency: %3uMHz    ",GetFreq(demo_param.freq_number));
			LcdWriteLine(LCD_LINE_2,lcd_data);	

		break;

		case sMenuPN9StartScreen:
			LcdClearDisplay();
			memcpy(lcd_data,"      PN9 TEST       ",21);
			printf("PN9 test\r\n>"); 
			LcdWriteLine(LCD_LINE_4,lcd_data);
			GetDataRate( rf_setting, &string[0]);		
			string[3] = 0;
			sprintf(lcd_data,"Datarate: %skbps   ",string);
			LcdWriteLine(LCD_LINE_1,lcd_data);	
			sprintf(lcd_data,"Frequency: %3uMHz    ",GetFreq(demo_param.freq_number));
			LcdWriteLine(LCD_LINE_2,lcd_data);	

		break;
#endif

		default:
		break;
	}
	return 0;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void MenuReportPer(uint16 sent_packets, uint16 received_packets, float per
  +
  + DESCRIPTION:    reports the PER into the LCD and UART
  +	
  + RETURN:         None
  +
  +	INPUT:			number of sent / received packets, calculated PER, ARSSI on the antennas
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void MenuReportPer(uint16 sent_packets, uint16 received_packets, float per)
{
uint8 lcd_data[22];

	printf(" %03u packet sent, %03u ACK received, ",sent_packets,received_packets);
	printf("PER:%3.2f\r\n",per);
	sprintf(lcd_data,"                     ");
	sprintf(lcd_data,"%03u/%03u  PER:%3.2f%%",received_packets,sent_packets,per);
	LcdWriteLine(LCD_LINE_6,lcd_data);
	//print the RSSI result to the screen	
	PrintRssiToLcd();
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void MenuReportTestPer(uint16 sent_packets, uint16 received_packets, float per, uint8 rssi1, uint8 rssi2)
  +
  + DESCRIPTION:    reports the PER into the LCD and UART
  +	
  + RETURN:         None
  +
  +	INPUT:			number of sent / received packets, calculated PER, ARSSI on the antennas
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
#ifdef TEST_FUNCTIONS_ENABLED
void MenuReportTestPer(uint16 sent_packets, uint16 received_packets, float per)
{
	uint8 lcd_data[22],fPacketReceived,rssi1,rssi2;

	fPacketReceived = RFGetRssiResult(&rssi1, &rssi2);
	printf("TR:%05u MP:%05u RSSI:",sent_packets,(sent_packets - received_packets));
	if( fPacketReceived == RF_OK )
	{
		if(TestcardData.ant_type == ANT_DIVERSITY)
		{
			printf("%03u %03u",rssi1,rssi2);
		}
		else
		{
			printf("%03u",rssi1);
		}
	}
	else
	{		
		if(TestcardData.ant_type == ANT_DIVERSITY)
		{
			printf("xxx xxx");
		}
		else
		{
			printf("xxx");
		}
	}
	printf(" PER:%3.2f\r\n",per);
	//write to the LCD
	sprintf(lcd_data,"TR:%05u   MP:%05u  ",sent_packets,(sent_packets - received_packets));
	LcdWriteLine(LCD_LINE_5,lcd_data);
	sprintf(lcd_data,"                     ");
	sprintf(lcd_data,"PER:%3.2f%%",per);
	LcdWriteLine(LCD_LINE_6,lcd_data);
}
#endif

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void GetDataRate(RF_SAMPLE_SETTINGS datarate, uint8 * string)
  +
  + DESCRIPTION:    it gives back the datarate in string format
  +	
  + RETURN:         string
  +
  +	INPUT:			data rate
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void GetDataRate(RF_SAMPLE_SETTINGS datarate, uint8 * string)
{
	switch( datarate)
	{
		case DR2400BPS_DEV36KHZ: 		memcpy(string,"2.4",3); break;
		case DR4800BPS_DEV45KHZ: 		memcpy(string,"4.8",3); break;
		case DR9600BPS_DEV45KHZ:		memcpy(string,"9.6",3); break;
		case DR10KBPS_DEV5KHZ_MOD1: 	memcpy(string," 10",3); break;
		case DR20KBPS_DEV10KHZ_MOD1:	memcpy(string," 20",3); break;
		case DR40KBPS_DEV20KHZ_MOD1: 	memcpy(string," 40",3); break;
		case DR50KBPS_DEV25KHZ_MOD1: 	memcpy(string," 50",3); break;
		case DR100KBPS_DEV50KHZ_MOD1: 	memcpy(string,"100",3); break;
		case DR125KBPS_DEV62KHZ5_MOD1: 	memcpy(string,"125",3); break;
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void PrintRssiToLcd(void)
  +
  + DESCRIPTION:    it prints the RSS result(s) to the LCD
  +	
  + RETURN:         None
  +
  +	INPUT:			None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void PrintRssiToLcd(void)
{
	uint8 lcd_data[22],rssi1, rssi2, fPacketReceived,bat;

	//get RSSI value(s)
	fPacketReceived = RFGetRssiResult(&rssi1, &rssi2);	

	//print to the screen
	sprintf(&lcd_data[0],"       RSSI:         ");
	if( TestcardData.ant_type == ANT_DIVERSITY )
	{
		if( fPacketReceived == RF_OK)
		{
			sprintf(&lcd_data[12],"%03u %03u  ",rssi1,rssi2);
		}
		else
		{
			sprintf(&lcd_data[12],"--- ---  ");
		}
	}
	else
	{
		if( fPacketReceived == RF_OK)
		{
			sprintf(&lcd_data[12],"%03u      ",rssi1);
		}
		else
		{
			sprintf(&lcd_data[12],"---      ");
		}
	}
	LcdWriteLine(LCD_LINE_3,lcd_data);
}

