/*
** ============================================================================
**
** FILE
**  $Id: pic18_timers.h,v 1.0 2007/06/20 krk Exp apache $
**
** DESCRIPTION
**  Contains the PIC18 timer specific declarations
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#ifndef PIC18_TIMERS_H
#define PIC18_TIMERS_H

#include "PIC.h"

                /* ======================================= *
                 *     T Y P E   D E C L A R A T I O N     *
                 * ======================================= */


                /* ======================================= *
                 *    PIC dependend 	M A C R O S        *
                 * ======================================= */
#define	EnableTmr0It()				TMR0IE = 1; TMR0IF = 0
#define	DisableTmr0It()				TMR0IE = 0; TMR0IF = 0
#define StopTmr0()					T0CON = 0;  TMR0IF = 0; TMR0IE = 0
#define EnableTmr1It()				TMR1IE = 1; TMR1IF = 0
#define DisableTmr1It()				TMR1IE = 0; TMR1IF = 0
#define StopTmr1()					T1CON = 0;  TMR1IF = 0; TMR1IE = 1
#define EnableTmr2It()				TMR2IE = 1; TMR2IF = 0
#define DisableTmr2It()				TMR2IE = 0; TMR2IF = 0
#define StopTmr2()					T2CON = 0;  TMR2IF = 0; TMR2IE = 1
#define EnableTmr3It()				TMR3IE = 1; TMR3IF = 0
#define DisableTmr3It()				TMR3IE = 0; TMR3IF = 0
#define StopTmr3()					T3CON = 0;  TMR3IF = 0; TMR3IE = 1

                /* ======================================= *
                 *          D E F I N I T I O N S          *
                 * ======================================= */
#define TMR0_8BIT					0x40
#define TMR0_16BIT					0x00
#define TMR0_ENABLE					0x80	//internal clock
#define TMR0_256					0x07
#define TMR0_128					0x06
#define TMR0_64						0x05
#define TMR0_32						0x04
#define TMR0_16						0x03
#define	TMR0_8						0x02
#define TMR0_4						0x01
#define TMR0_2						0x00
#define TMR0_1						0x08	

#define TMR1_INTERNAL_ENABLE		0x81	//16bit read, Fosc/4
#define TMR1_OSC_ENABLE				0xCF	//16bit read, T1 osc, non synchronized
#define TMR1_8						0x30
#define TMR1_4						0x20
#define TMR1_2						0x10
#define TMR1_1						0x00

#define TMR2_ENABLE					0x04
#define TMR2_PRE_1					0x00
#define TMR2_PRE_4					0x01
#define	TMR2_PRE_16					0x02
#define TMR2_POS_1					0x00
#define TMR2_POS_2					0x08
#define TMR2_POS_3					0x10
#define TMR2_POS_4					0x18
#define TMR2_POS_5					0x20
#define TMR2_POS_6					0x28
#define TMR2_POS_7					0x30
#define TMR2_POS_8					0x38
#define TMR2_POS_9					0x40
#define TMR2_POS_10					0x48
#define TMR2_POS_11					0x50
#define TMR2_POS_12					0x58
#define TMR2_POS_13					0x60
#define	TMR2_POS_14					0x68
#define TMR2_POS_15					0x70
#define TMR2_POS_16					0x78

#define TMR3_INTERNAL_ENABLE		0x81  	//16 bit read, Fosc/4
#define TMR3_OSC_ENABLE				0x87  	//16bit read, T1 osc, non synchronized			
#define TMR3_8						0x30
#define TMR3_4						0x20
#define TMR3_2						0x10
#define TMR3_1						0x00


                /* ======================================= *
                 *            V A R I A B L E S            *
                 * ======================================= */


                /* ======================================= *
                 *  F U N C T I O N   P R O T O T Y P E S  *
                 * ======================================= */
void StartTmr0(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable);
uint16 GetTmr0(uint8 operation_mode);
uint8 Tmr0Expired(void);
void StartTmr1(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable);
uint16 GetTmr1(void);
uint8 Tmr1Expired(void);
void StartTmr2(uint8 prescaler, uint8 period, uint8 it_enable);
uint8 GetTmr2(void);
uint8 Tmr2Expired(void);
void StartTmr3(uint8 operation_mode, uint8 prescaler, uint16 period, uint8 it_enable);
uint16 GetTmr3(void);
uint8 Tmr3Expired(void);
void StartSystemTimer(void);
void IncrementSystemTimer(void);
uint32 GetSystemTimer(void);
#endif
