/*
** ============================================================================
**
** FILE
**  $Id: uart.c,v 1.0 2007/06/20 krk Exp apache $
**
** DESCRIPTION
**  Contains the UART functions.
**
** CREATED
**  Integration Hungary Ltd
**
** COPYRIGHT
**  Copyright 2006 Integration Associates Inc.  All rights reserved.
**
** Permission to use, copy, modify, revise, translate, abridge, condense, expand, collect,
** compile, link, recast, distribute, transform or adapt this software is granted only by 
** specific written license from Integration Associates Inc.  Contact Integration Associates 
** at 110 Pioneer Way, Unit L, Mountain View, California 94041, Attention Sales & Marketing, 
** or call (650) 969-4100 for information about obtaining a license.
** 
** Any copy or modification made must satisfy the following conditions:
** 
** 1. Both the copyright notice and this permission notice appear in all copies of the software, 
**    derivative works or modified versions, and any portions thereof, and that both notices 
**    appear in supporting documentation.
** 
** 2. All copies of the software shall contain the following acknowledgement: "Portions of this 
**    software are used under license from Integration Associates Inc. and are copyrighted."
** 
** 3. Neither the name of Integration Associates Inc. nor any of its subsidiaries may be used 
**    to endorse or promote products derived from this software without specific prior written 
**    permission.
** 
** THIS SOFTWARE IS PROVIDED BY "AS IS" AND ALL WARRANTIES OF ANY KIND, INCLUDING THE IMPLIED 
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR USE, ARE EXPRESSLY DISCLAIMED.  THE DEVELOPER 
** SHALL NOT BE LIABLE FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.  
** THIS SOFTWARE MAY NOT BE USED IN PRODUCTS INTENDED FOR USE IN IMPLANTATION OR OTHER DIRECT 
** LIFE SUPPORT APPLICATIONS WHERE MALFUNCTION MAY RESULT IN THE DIRECT PHYSICAL HARM OR INJURY 
** TO PERSONS.  ALL SUCH IS USE IS EXPRESSLY PROHIBITED.
**   
** ============================================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "global_definitions.h"
#include "uart.h"
#include "pic18_timers.h"
#include "IA443x_demo.h"
#include "IA443x_test.h"
#include "IA443x_rf.h"
#ifdef USB_CONNECTION_SUPPORT
 #include "USB_connection.c"
#endif

/*------------------------------------------------------------------------*/
/*						GLOBAL variables								  */
/*------------------------------------------------------------------------*/
volatile uint8 fUartCommandReceived, UartRxPointer;		//flag to show if an UART command received
volatile uint8 UartRxBuffer[UART_RX_BUFFER_SIZE], UartRxMessage[UART_RX_BUFFER_SIZE];
volatile uint8 UartEcho;
uint16 SpiCommand;
uint16 ComParam1;
volatile uint8 fChangeParam;

extern volatile S_TEST sDemo;
extern volatile uint8 customer_name[CUSTOMER_NAME_SIZE];
#ifdef TEST_FUNCTIONS_ENABLED
 extern volatile uint8 rf_setting;
#endif

/*------------------------------------------------------------------------*/
/*						LOCAL function prototypes						  */
/*------------------------------------------------------------------------*/
void _PrintOk(void);
void _PrintWrongParam(void);

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void UartRxISR(void)
  +
  + DESCRIPTION:    UART character receive interrupt routine
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void UartRxISR(void)
{
	uint8 RxData;
	
	//get data from the register
	RxData = RCREG;
	if( (RxData != ' ') && (RxData != CHAR_LF) )
	{
		UartRxBuffer[UartRxPointer] = RxData;
	}
	else
	{
		return;
	}
	//is the buffer full, or was the last char ENTER?
  	if( (UartRxPointer == UART_RX_BUFFER_SIZE - 1) || (UartRxBuffer[UartRxPointer] == MESSAGE_END) )
    {
    	//Message received - last char should be NULL
    	UartRxBuffer[UartRxPointer] = 0;
		UartRxPointer = 0;
		strcpy((uint8*)UartRxMessage,(uint8*)UartRxBuffer);
		fUartCommandReceived = TRUE;
    }
  	else
    {
    	//if the last char was not back space, then inc the pointer
    	if( !(UartRxBuffer[UartRxPointer] == CHAR_BACKSPACE) )
      		UartRxPointer++;
    	else
      	{
     		if(UartRxPointer > 0)
        	UartRxPointer--;
      	}
    }
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void UartInit(void)
  +
  + DESCRIPTION:    initialize the UART peripheral
  +
  + RETURN:         None
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void UartInit(void)
{
	//enable UART peripheral
	UART_TX_DIR = 1;
	UART_RX_DIR = 1;
	TXSTA 		= UART_ASYNCHRON | UART_TX_8BIT | UART_TX_ENABLE | UART_HIGH_BAUD;
	RCSTA 		= UART_ENABLE | UART_RX_8BIT | UART_RX_ENABLE;
	BAUDCON 	= UART_16BIT_BAUD;

	 //baud rate is set to 19.2kbps
	 SPBRGH		= 0;
	 SPBRG		= 103;	//19.2kbps @ 8MHz

	//set default value for the variable
	UartRxPointer = 0;
	fUartCommandReceived = FALSE;

	//enable RX interrupt
	EnablePeripheralIt();
	EnableUartRxIt();
	TXIE = 0;
	UartEcho = 1;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  uint8 putch(uint8 ch)
  +
  + DESCRIPTION:    send a character through the UART
  +
  + RETURN:         None
  +
  + NOTES:          UartInit() has to be call before
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void putch(char ch)
{	
	while( TXIF == 0 );
	//send next char
	TXREG = ch;
}


/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME:  void CommandInterpreter(void)
  +
  + DESCRIPTION:    it check the received string and execute the operation
  +					if it is a valid UART command
  +
  + RETURN:         None
  +
  + NOTES:          UartInit() has to be call before
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void UartCommandInterpreter(void)
{
	uint8 temp8,i;
	uint16 temp16, ReceivedWord;


	/*ENTER*/
	if( UartRxMessage[0] == 0 )
	{
		printf("\r\n>");
		return;
	}

	/*SET CUSTOMER NAME*/
	if( (memcmp(&UartRxMessage[0], "scn:", 4) == 0) || (memcmp(&UartRxMessage[0], "SCN:", 4) == 0) )
	{//set customer name	
		i = 0;
		while( (i < (CUSTOMER_NAME_SIZE - 1)) && (UartRxMessage[i + 4] != MESSAGE_END) )
		{
			if( UartRxMessage[i + 4] == '_' )
			{
				customer_name[i] = ' ';
			}
			else
			{
				customer_name[i] = UartRxMessage[i + 4];
			}
			EEPROM_WRITE(i,customer_name[i]);
			i++;
		};
		customer_name[i] = 0;
		EEPROM_WRITE(i,customer_name[i]);
		printf("\r\n customer name: %s",customer_name);
		_PrintOk();
		return;
	}

#ifdef USB_CONNECTION_SUPPORT
	if( USBConnectionSupport( (uint8*)&UartRxMessage[0] ) )
	{
		_PrintOk();
		return;
	}
#endif

#ifdef TEST_FUNCTIONS_ENABLED
	if( (memcmp(&UartRxMessage[0], "demo", 4) == 0) || (memcmp(&UartRxMessage[0], "DEMO", 4) == 0) )
	{//start demo
		sDemo = DEMO;
		DemoInit();
		TestInit();
		_PrintOk();
		return;
	}

	if( (memcmp(&UartRxMessage[0], "per", 3) == 0) || (memcmp(&UartRxMessage[0], "PER", 3) == 0) )
	{//start triggered PER test
		temp8 = UartRxMessage[3] - 0x30; 
		if( temp8 < NMBR_OF_SAMPLE_SETTING )
		{
			rf_setting = temp8;
			sDemo = PER;
			DemoInit();
			TestInit();
			_PrintOk();
			return;
		}
	}

	if( (memcmp(&UartRxMessage[0], "ber", 3) == 0) || (memcmp(&UartRxMessage[0], "BER", 3) == 0) )
	{//start BER test (direct receive mode)
		temp8 = UartRxMessage[3] - 0x30; 
		if( temp8 < NMBR_OF_SAMPLE_SETTING )
		{
			rf_setting = temp8;
			sDemo = BER;
			DemoInit();
			TestInit();
			_PrintOk();
			return;
		}
	}

	if( (memcmp(&UartRxMessage[0], "cw", 2) == 0) || (memcmp(&UartRxMessage[0], "CW", 2) == 0) )
	{//start CW mode
		temp8 = UartRxMessage[2] - 0x30; 
		if( temp8 < NMBR_OF_SAMPLE_SETTING )
		{
			rf_setting = temp8;
			sDemo = CW;
			DemoInit();
			TestInit();
			_PrintOk();
			return;
		}
	}

	if( (memcmp(&UartRxMessage[0], "pn9", 3) == 0) || (memcmp(&UartRxMessage[0], "PN9", 3) == 0) )
	{//start PN9 mode
		temp8 = UartRxMessage[3] - 0x30; 
		if( temp8 < NMBR_OF_SAMPLE_SETTING )
		{
			rf_setting = temp8;
			sDemo = PN9;
			DemoInit();
			TestInit();
			_PrintOk();
			return;
		}
	}
#endif

	/*INVALID COMMAND*/
	_PrintWrongParam();
}



/*------------------------------------------------------------------------*/
/*						LOCAL functions 								  */
/*------------------------------------------------------------------------*/
/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME: void _PrintOk(void)
  +
  + DESCRIPTION: print OK and new prompt  
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void _PrintOk(void)
{
	switch(UartEcho)
	{
		case 0:
		case 1:
		case 2:
		 printf("\n\rOK\n\r>"); 
		break;

		default:
	     printf("\n\r>");
		break;
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
  +
  + FUNCTION NAME: void _PrintWrongParam(void)
  +
  + DESCRIPTION: print worng parameter message  
  +
  + RETURN:         
  +
  + NOTES:          
  +
  +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/
void _PrintWrongParam(void)
{
	printf("\n\rERROR\n\r>");
}




